@@ -1,4 +1,4 @@
-;;; vietnamese.el --- Support for Vietnamese
+;;; vietnamese.el --- support for Vietnamese -*- coding: iso-2022-7bit; -*-
 
 ;; Copyright (C) 1995 Electrotechnical Laboratory, JAPAN.
 ;; Licensed to the Free Software Foundation.
@@ -28,8 +28,6 @@
 
 ;;; Code:
 
-(eval-when-compile
-
 (defvar viet-viscii-decode-table
   [;; VISCII is a full 8-bit code.
    0 1 ?,2F(B 3 4 ?,2G(B ?,2g(B 7 8 9 10 11 12 13 14 15
@@ -48,24 +46,12 @@
    ?,2p(B ?,1Q(B ?,2r(B ?,2s(B ?,2t(B ?,1U(B ?,1V(B ?,1W(B ?,1X(B ?,2y(B ?,2z(B ?,1[(B ?,1\(B ?,2}(B ?,1^(B ?,1_(B
    ?,1`(B ?,1a(B ?,1b(B ?,1c(B ?,1d(B ?,1e(B ?,1f(B ?,1g(B ?,1h(B ?,1i(B ?,1j(B ?,1k(B ?,1l(B ?,1m(B ?,1n(B ?,1o(B
    ?,1p(B ?,1q(B ?,1r(B ?,1s(B ?,1t(B ?,1u(B ?,1v(B ?,1w(B ?,1x(B ?,1y(B ?,1z(B ?,1{(B ?,1|(B ?,1}(B ?,1~(B ?,2f(B ]
-  "Vietnamese VISCII encoding table.")
+  "Vietnamese VISCII decoding table.")
 
-(defvar viet-viscii-encode-table
-  (let ((table-lower (make-vector 128 0))
-	(table-upper (make-vector 128 0))
-	(i 0)
-	char-component)
-    (while (< i 256)
-      (setq char-component (split-char (aref viet-viscii-decode-table i)))
-      (cond ((eq (car char-component) 'vietnamese-viscii-lower)
-	     (aset table-lower (nth 1 char-component) i))
-	    ((eq (car char-component) 'vietnamese-viscii-upper)
-	     (aset table-upper (nth 1 char-component) i)))
-      (setq i (1+ i)))
-    (cons table-lower table-upper))
-  "Vietnamese VISCII decoding table.
-Cons of tables for decoding lower-case chars and upper-case characterss.
-Both tables are indexed by the position code of Vietnamese characters.")
+(let ((table (make-translation-table-from-vector viet-viscii-decode-table)))
+  (define-translation-table 'viet-viscii-nonascii-translation-table table)
+  (define-translation-table 'viet-viscii-encode-table
+    (char-table-extra-slot table 0)))
 
 (defvar viet-vscii-decode-table
   [;; VSCII is a full 8-bit code.
@@ -79,69 +65,35 @@ Both tables are indexed by the position code of Vietnamese characters.")
    112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 
    ?,2`(B ?,2d(B ?,2c(B ?,2a(B ?,2U(B ?,2#(B ?,2'(B ?,2h(B ?,2k(B ?,2((B ?,2i(B ?,2)(B ?,2.(B ?,2l(B ?,2o(B ?,2n(B
    ?,2m(B ?,28(B ?,2r(B ?,2v(B ?,2u(B ?,2s(B ?,2w(B ?,25(B ?,26(B ?,27(B ?,2^(B ?,2>(B ?,2~(B ?,2y(B ?,2|(B ?,2{(B
-   160 ?,2e(B ?,2b(B ?,2j(B ?,2t(B ?,2=(B ?,2_(B ?,2p(B ?,1e(B ?,1b(B ?,1j(B ?,1t(B ?,1>(B ?,1y(B ?,1p(B ?,2"(B
+   160 ?,2e(B ?,2b(B ?,2j(B ?,2t(B ?,2=(B ?,2_(B ?,2p(B ?,1e(B ?,1b(B ?,1j(B ?,1t(B ?,1=(B ?,1y(B ?,1p(B ?,2"(B
    192 193 194 195 196 ?,1`(B ?,1d(B ?,1c(B ?,1a(B ?,1U(B ?,2F(B ?,1"(B ?,1F(B ?,1G(B ?,1!(B ?,2G(B
    ?,2!(B ?,2%(B ?,2&(B ?,2g(B ?,2%(B ?,2+(B ?,1#(B ?,1%(B ?,1&(B ?,1g(B ?,1$(B ?,1'(B ?,1h(B ?,2,(B ?,1k(B ?,1((B
    ?,1i(B ?,1)(B ?,1+(B ?,1,(B ?,1-(B ?,1*(B ?,1.(B ?,1l(B ?,1o(B ?,2-(B ?,2*(B ?,20(B ?,1n(B ?,1m(B ?,18(B ?,1r(B
    ?,21(B ?,1v(B ?,1u(B ?,1s(B ?,1w(B ?,10(B ?,11(B ?,12(B ?,1/(B ?,15(B ?,16(B ?,17(B ?,1^(B ?,1>(B ?,1~(B ?,1y(B
    ?,22(B ?,1|(B ?,1{(B ?,1z(B ?,1x(B ?,1W(B ?,1X(B ?,1f(B ?,1Q(B ?,1q(B ?,1O(B ?,1V(B ?,1[(B ?,1}(B ?,1\(B ?,2/(B]
-  "Vietnamese VSCII code table.")
+  "Vietnamese VSCII decoding table.")
 
-(defvar viet-vscii-encode-table
-  (let ((table-lower (make-vector 128 0))
-	(table-upper (make-vector 128 0))
-	(i 0)
-	char-component)
-    (while (< i 256)
-      (setq char-component (split-char (aref viet-vscii-decode-table i)))
-      (cond ((eq (car char-component) 'vietnamese-viscii-lower)
-	     (aset table-lower (nth 1 char-component) i))
-	    ((eq (car char-component) 'vietnamese-viscii-upper)
-	     (aset table-upper (nth 1 char-component) i)))
-      (setq i (1+ i)))
-    (cons table-lower table-upper))
-  "Vietnamese VSCII decoding table.
-Cons of tables for decoding lower-case chars and upper-case characterss.
-Both tables are indexed by the position code of Vietnamese characters.")
-
-)
+(let ((table (make-translation-table-from-vector viet-vscii-decode-table)))
+  (define-translation-table 'viet-vscii-nonascii-translation-table table)
+  (define-translation-table 'viet-vscii-encode-table
+    (char-table-extra-slot table 0)))
 
 (define-ccl-program ccl-decode-viscii
   `(3
-    ((read r0)
-     (loop
-      (write-read-repeat r0 ,viet-viscii-decode-table))
-     ))
+    ((loop
+      (r0 = 0)
+      (read r1)
+      (translate-character viet-viscii-nonascii-translation-table r0 r1)
+      (write-multibyte-character r0 r1)
+      (repeat))))
   "CCL program to decode VISCII 1.1")
 
-;; Multibyte form of a Vietnamese character is as follows (3-byte):
-;;   LEADING-CODE-PRIVATE-11 LEADING-CODE-EXTENDED-11 POSITION-CODE
-;; where LEADING-CODE-EXTENDED-11 for Vietnamese is
-;; `vietnamese-viscii-lower' or `vietnamese-viscii-upper'.
-
 (define-ccl-program ccl-encode-viscii
   `(1
-     ((read r0)
-      (loop
-       (if (r0 < 128)
-	   ;; ASCII
-	   (write-read-repeat r0)
-	 ;; not ASCII
-	 (if (r0 != ,leading-code-private-11)
-	     ;; not Vietnamese
-	     (write-read-repeat r0)
-	   ((read-if (r0 == ,(charset-id 'vietnamese-viscii-lower))
-	     (;; Vietnamese lower
-	      (read r0)
-	      (r0 -= 128)
-	      (write-read-repeat r0 ,(car viet-viscii-encode-table)))
-	     (if (r0 == ,(charset-id 'vietnamese-viscii-upper))
-		 (;; Vietnamese upper
-		  (read r0)
-		  (r0 -= 128)
-		  (write-read-repeat r0 ,(cdr viet-viscii-encode-table)))
-	       ;; not Vietnamese
-	       (write-read-repeat r0)))))))))
+    ((loop
+      (read-multibyte-character r0 r1)
+      (translate-character viet-viscii-encode-table r0 r1)
+      (write-repeat r1))))
   "CCL program to encode VISCII 1.1")
 
 (define-ccl-program ccl-encode-viscii-font
@@ -149,43 +101,25 @@ Both tables are indexed by the position code of Vietnamese characters.")
     ;; In:  R0:vietnamese-viscii-lower/vietnamese-viscii-upper
     ;;      R1:position code
     ;; Out: R1:font code point
-    (if (r0 == ,(charset-id 'vietnamese-viscii-lower))
-	(r1 = r1 ,(car viet-viscii-encode-table))
-      (r1 = r1 ,(cdr viet-viscii-encode-table)))
-    )
+    (translate-character viet-viscii-encode-table r0 r1))
   "CCL program to encode Vietnamese chars to VISCII 1.1 font")
 
 (define-ccl-program ccl-decode-vscii
   `(3
-    ((read r0)
-     (loop
-      (write-read-repeat r0 ,viet-vscii-decode-table))
-     ))
+    ((loop
+      (r0 = 0)
+      (read r1)
+      (translate-character viet-vscii-nonascii-translation-table r0 r1)
+      (write-multibyte-character r0 r1)
+      (repeat))))
   "CCL program to decode VSCII-1.")
 
 (define-ccl-program ccl-encode-vscii
   `(1
-    ((read r0)
-     (loop
-      (if (r0 < 128)
-	  ;; ASCII
-	  (write-read-repeat r0)
-	;; not ASCII 
-	(if (r0 != ,leading-code-private-11)
-	    ;; not Vietnamese
-	    (write-read-repeat r0)
-	  (read-if (r0 == ,(charset-id 'vietnamese-viscii-lower))
-		   (;; Vietnamese lower
-		    (read r0)
-		    (r0 -= 128)
-		    (write-read-repeat r0 ,(car viet-vscii-encode-table)))
-		   (if (r0 == ,(charset-id 'vietnamese-viscii-upper))
-		       (;; Vietnamese upper
-			(read r0)
-			(r0 -= 128)
-			(write-read-repeat r0 ,(cdr viet-viscii-encode-table)))
-		     ;; not Vietnamese
-		     (write-read-repeat r0))))))))
+    ((loop
+      (read-multibyte-character r0 r1)
+      (translate-character viet-vscii-encode-table r0 r1)
+      (write-repeat r1))))
   "CCL program to encode VSCII-1.")
 
 (define-ccl-program ccl-encode-vscii-font
@@ -193,52 +127,68 @@ Both tables are indexed by the position code of Vietnamese characters.")
     ;; In:  R0:vietnamese-viscii-lower/vietnamese-viscii-upper
     ;;      R1:position code
     ;; Out: R1:font code point
-    (if (r0 == ,(charset-id 'vietnamese-viscii-lower))
-	(r1 = r1 ,(car viet-vscii-encode-table))
-      (r1 = r1 ,(cdr viet-vscii-encode-table)))
-    )
+    (translate-character viet-vscii-encode-table r0 r1))
   "CCL program to encode Vietnamese chars to VSCII-1 font.")
 
 
 (make-coding-system
  'vietnamese-viscii 4 ?V
  "8-bit encoding for Vietnamese VISCII 1.1 (MIME:VISCII)"
- (cons ccl-decode-viscii ccl-encode-viscii))
+ '(ccl-decode-viscii . ccl-encode-viscii)
+ '((safe-charsets ascii vietnamese-viscii-lower vietnamese-viscii-upper)
+   (mime-charset . viscii)
+   (valid-codes (0 . 255))))
 
 (define-coding-system-alias 'viscii 'vietnamese-viscii)
 
 (make-coding-system
  'vietnamese-vscii 4 ?v
  "8-bit encoding for Vietnamese VSCII-1"
- (cons ccl-decode-vscii ccl-encode-vscii))
+ '(ccl-decode-vscii . ccl-encode-vscii)
+ '((safe-charsets ascii vietnamese-viscii-lower vietnamese-viscii-upper)
+   (valid-codes (0 . 255))))
 
 (define-coding-system-alias 'vscii 'vietnamese-vscii)
 
 (make-coding-system
  'vietnamese-viqr 0 ?q
  "Vietnamese latin transcription (VIQR)"
- nil)
-(put 'vietnamese-viqr 'post-read-conversion 'viqr-post-read-conversion)
-(put 'vietnamese-viqr 'pre-write-conversion 'viqr-pre-write-conversion)
+ nil
+ '((safe-charsets ascii vietnamese-viscii-lower vietnamese-viscii-upper)
+   (post-read-conversion . viqr-post-read-conversion)
+   (pre-write-conversion . viqr-pre-write-conversion)
+   (charset-origin-alist
+    (vietnamese-viscii-lower "VISCII" viet-encode-viscii-char)
+    (vietnamese-viscii-upper "VISCII" viet-encode-viscii-char))))
+
 
 (define-coding-system-alias 'viqr 'vietnamese-viqr)
 
 (setq font-ccl-encoder-alist
-      (cons (cons "viscii" ccl-encode-viscii-font) font-ccl-encoder-alist))
+      (cons '("viscii" . ccl-encode-viscii-font) font-ccl-encoder-alist))
 
 (setq font-ccl-encoder-alist
-      (cons (cons "vscii" ccl-encode-vscii-font) font-ccl-encoder-alist))
+      (cons '("vscii" . ccl-encode-vscii-font) font-ccl-encoder-alist))
 
 (set-language-info-alist
- "Vietnamese" '((setup-function . setup-vietnamese-environment)
-		(charset . (vietnamese-viscii-lower
-			    vietnamese-viscii-upper))
-		(coding-system . (vietnamese-viscii vietnamese-vscii
-				  vietnamese-viqr))
+ "Vietnamese" `((charset vietnamese-viscii-lower vietnamese-viscii-upper)
+		(nonascii-translation
+		 . ,(get 'viet-viscii-nonascii-translation-table
+			 'translation-table))
+		(coding-system vietnamese-viscii vietnamese-vscii
+			       vietnamese-viqr)
+		(coding-priority vietnamese-viscii)
+		(input-method . "vietnamese-viqr")
+		(unibyte-display . vietnamese-viscii)
+		(features viet-util)
 		(sample-text . "Vietnamese (Ti,1*(Bng Vi,1.(Bt)	Ch,1`(Bo b,1U(Bn")
 		(documentation . "\
-For Vietnamese, Emacs uses special charasets internally.
-They can be decoded from and encoded to VISCC, VSCII, and VIQR.")
+For Vietnamese, Emacs uses special charsets internally.
+They can be decoded from and encoded to VISCC, VSCII, and VIQR.
+Current setting puts higher priority to the coding system VISCII than VSCII.
+If you prefer VSCII, please do: (prefer-coding-system 'vietnamese-vscii)")
 		))
 
+(provide 'vietnamese)
+
 ;;; vietnamese.el ends here
