@@ -1,4 +1,4 @@
-;;; macros.el --- non-primitive commands for keyboard macros.
+;;; macros.el --- non-primitive commands for keyboard macros
 
 ;; Copyright (C) 1985, 86, 87, 92, 94, 95 Free Software Foundation, Inc.
 
@@ -43,7 +43,7 @@ Such a \"function\" cannot be called from Lisp, but it is a valid editor command
   (and (fboundp symbol)
        (not (stringp (symbol-function symbol)))
        (not (vectorp (symbol-function symbol)))
-       (error "Function %s is already defined and not a keyboard macro."
+       (error "Function %s is already defined and not a keyboard macro"
 	      symbol))
   (if (string-equal symbol "")
       (error "No command name given"))
@@ -150,6 +150,8 @@ use this command, and then save the file."
 		       (setq mods (cdr mods)))
 		     (cond ((= char ?\\)
 			    (insert "\\\\"))
+			   ((= char ?\;)
+			    (insert "\\;"))
 			   ((= char 127)
 			    (insert "\\C-?"))
 			   ((< char 127)
@@ -182,20 +184,20 @@ Your options are: \\<query-replace-map>
 \\[recenter]	Redisplay the screen, then ask again.
 \\[edit]	Enter recursive edit; ask again when you exit from that."
   (interactive "P")
-  (or executing-macro
+  (or executing-kbd-macro
       defining-kbd-macro
       (error "Not defining or executing kbd macro"))
   (if flag
-      (let (executing-macro defining-kbd-macro)
+      (let (executing-kbd-macro defining-kbd-macro)
 	(recursive-edit))
-    (if (not executing-macro)
+    (if (not executing-kbd-macro)
 	nil
       (let ((loop t)
 	    (msg (substitute-command-keys
 		  "Proceed with macro?\\<query-replace-map>\
  (\\[act], \\[skip], \\[exit], \\[recenter], \\[edit]) ")))
 	(while loop
-	  (let ((key (let ((executing-macro nil)
+	  (let ((key (let ((executing-kbd-macro nil)
 			   (defining-kbd-macro nil))
 		       (message "%s" msg)
 		       (read-event)))
@@ -206,14 +208,14 @@ Your options are: \\<query-replace-map>
 		   (setq loop nil))
 		  ((eq def 'skip)
 		   (setq loop nil)
-		   (setq executing-macro ""))
+		   (setq executing-kbd-macro ""))
 		  ((eq def 'exit)
 		   (setq loop nil)
-		   (setq executing-macro t))
+		   (setq executing-kbd-macro t))
 		  ((eq def 'recenter)
 		   (recenter nil))
 		  ((eq def 'edit)
-		   (let (executing-macro defining-kbd-macro)
+		   (let (executing-kbd-macro defining-kbd-macro)
 		     (recursive-edit)))
 		  ((eq def 'quit)
 		   (setq quit-flag t))
@@ -279,7 +281,7 @@ and then select the region of un-tablified names and use
   (or macro
       (progn
 	(if (null last-kbd-macro)
-	    (error "No keyboard macro has been defined."))
+	    (error "No keyboard macro has been defined"))
 	(setq macro last-kbd-macro)))
   (save-excursion
     (let ((end-marker (progn
@@ -303,4 +305,6 @@ and then select the region of un-tablified names and use
 
 ;;;###autoload (define-key ctl-x-map "q" 'kbd-macro-query)
 
+(provide 'macros)
+
 ;;; macros.el ends here
