@@ -1,8 +1,8 @@
-;;;_ landmark.el --- neural-network robot that learns landmarks
+;;; landmark.el --- neural-network robot that learns landmarks
 
-;; Copyright (c) 1996, 1997 Free Software Foundation, Inc.
+;; Copyright (c) 1996, 1997, 2000 Free Software Foundation, Inc.
 
-;; Author: Terrence Brannon <brannon@rana.usc.edu>
+;; Author: Terrence Brannon (was: <brannon@rana.usc.edu>)
 ;; Created: December 16, 1996 - first release to usenet
 ;; Keywords: gomoku neural network adaptive search chemotaxis
 
@@ -30,7 +30,7 @@
 ;; Boston, MA 02111-1307, USA.
 
 
-;;;_* Commentary
+;;; Commentary:
 ;;; Lm is a relatively non-participatory game in which a robot
 ;;; attempts to maneuver towards a tree at the center of the window
 ;;; based on unique olfactory cues from each of the 4 directions. If
@@ -54,20 +54,21 @@
 ;;; a single move, one moves east,west and south, then both east and
 ;;; west will be improved when they shouldn't
 
-;;; For further references see
-;;; http://rana.usc.edu:8376/~brannon/warez/yours-truly/lm/
 ;;; Many thanks to Yuri Pryadkin (yuri@rana.usc.edu) for this
 ;;; concise problem description.
 
-;;;_* Provide
-
-(provide 'lm)
-
 ;;;_* Require
-(require 'cl)
+(eval-when-compile (require 'cl))
 
 ;;;_* From Gomoku
 
+;;; Code:
+
+(defgroup lm nil
+  "Neural-network robot that learns landmarks."
+  :prefix "lm-"
+  :group 'games)
+
 ;;;_ +  THE BOARD.
 
 ;; The board is a rectangular grid. We code empty squares with 0, X's with 1
@@ -154,8 +155,10 @@
 
 ;;;_ +  LM MODE AND KEYMAP.
 
-(defvar lm-mode-hook nil
-  "If non-nil, its value is called on entry to Lm mode.")
+(defcustom lm-mode-hook nil
+  "If non-nil, its value is called on entry to Lm mode."
+  :type 'hook
+  :group 'lm)
 
 (defvar lm-mode-map nil
   "Local keymap to use in Lm mode.")
@@ -213,27 +216,30 @@
 (defvar lm-emacs-won ()
   "*For making font-lock use the winner's face for the line.")
 
-(defvar lm-font-lock-O-face
-  (if window-system
+(defvar lm-font-lock-face-O
+  (if (display-color-p)
       (list (facemenu-get-face 'fg:red) 'bold))
   "*Face to use for Emacs' O.")
 
-(defvar lm-font-lock-X-face
-  (if window-system
+(defvar lm-font-lock-face-X
+  (if (display-color-p)
       (list (facemenu-get-face 'fg:green) 'bold))
   "*Face to use for your X.")
 
 (defvar lm-font-lock-keywords
-  '(("O" . lm-font-lock-O-face)
-    ("X" . lm-font-lock-X-face)
+  '(("O" . lm-font-lock-face-O)
+    ("X" . lm-font-lock-face-X)
     ("[-|/\\]" 0 (if lm-emacs-won
-		     lm-font-lock-O-face
-		   lm-font-lock-X-face)))
+		     lm-font-lock-face-O
+		   lm-font-lock-face-X)))
   "*Font lock rules for Lm.")
 
 (put 'lm-mode 'front-sticky
      (put 'lm-mode 'rear-nonsticky '(intangible)))
 (put 'lm-mode 'intangible 1)
+;; This one is for when they set view-read-only to t: Landmark cannot
+;; allow View Mode to be activated in its buffer.
+(put 'lm-mode 'mode-class 'special)
 
 (defun lm-mode ()
   "Major mode for playing Lm against Emacs.
@@ -890,9 +896,11 @@ If the game is finished, this command requests for another game."
 			      ((= value 5) ?W)
 			      ((= value 6) ?^)))
 
-    (and window-system
-	 (zerop value)
-	 (put-text-property (1- (point)) (point) 'mouse-face 'highlight))
+    (and (zerop value)
+	 (add-text-properties (1- (point)) (point)
+			      '(mouse-face highlight
+				help-echo "\
+mouse-1: get robot moving, mouse-2: play on this square")))
     (delete-char 1)
     (backward-char 1))
   (sit-for 0))	; Display NOW
@@ -931,9 +939,9 @@ If the game is finished, this command requests for another game."
 		      (goto-char (point-max))))
 	       (setq point (point))
 	       (insert ?=)
-	       (if window-system
-		   (put-text-property point (point)
-				      'mouse-face 'highlight)))
+	       (add-text-properties point (point)
+				    '(mouse-face highlight help-echo "\
+mouse-1: get robot moving, mouse-2: play on this square")))
 	     (> (setq i (1- i)) 0))
       (if (= i (1- m))
 	  (setq opoint point))
@@ -1089,8 +1097,6 @@ If the game is finished, this command requests for another game."
   (move-to-column (+ lm-x-offset
 		     (* lm-square-width (1- lm-board-width)))))
 
-(provide 'lm)
-
 
 ;;;_ + Simulation variables
 
@@ -1118,17 +1124,19 @@ this program to add a random element to the way moves were made.")
 
 ;;;_* Terry's mods to create lm.el
 
-;;;_ + Debugging things
-(setq debug-on-error t)
 ;;;(setq lm-debug nil)
 (defvar lm-debug nil
   "If non-nil, debugging is printed.")
-(defvar lm-one-moment-please nil
+(defcustom lm-one-moment-please nil
   "If non-nil, print \"One moment please\" when a new board is generated.
 The drawback of this is you don't see how many moves the last run took
-because it is overwritten by \"One moment please\".")
-(defvar lm-output-moves t
-  "If non-nil, output number of moves so far on a move-by-move basis.")
+because it is overwritten by \"One moment please\"."
+  :type 'boolean
+  :group 'lm)
+(defcustom lm-output-moves t
+  "If non-nil, output number of moves so far on a move-by-move basis."
+  :type 'boolean
+  :group 'lm)
 
 
 (defun lm-weights-debug ()
@@ -1150,7 +1158,7 @@ because it is overwritten by \"One moment please\".")
 ;;(setq direction 'lm-n)
 ;;(get 'lm-n 'lm-s)
 (defun lm-nslify-wts-int (direction)
-  (mapcar '(lambda (target-direction)
+  (mapcar (lambda (target-direction)
 	     (get direction target-direction))
 	  lm-directions))
 
@@ -1163,7 +1171,7 @@ because it is overwritten by \"One moment please\".")
 		  (eval (cons 'max l)) (eval (cons 'min l))))))
 
 (defun lm-print-wts-int (direction)
-  (mapc '(lambda (target-direction)
+  (mapc (lambda (target-direction)
 	     (insert (format "%S %S %S "
 			      direction
 			      target-direction
@@ -1230,14 +1238,14 @@ because it is overwritten by \"One moment please\".")
     (set-buffer "*lm-blackbox*")
     (insert "==============================\n")
     (insert "I smell: ")
-    (mapc '(lambda (direction)
+    (mapc (lambda (direction)
 	       (if (> (get direction 'smell) 0)
 		   (insert (format "%S " direction))))
 	    lm-directions)
     (insert "\n")
 
     (insert "I move: ")
-    (mapc '(lambda (direction)
+    (mapc (lambda (direction)
 	       (if (> (get direction 'y_t) 0)
 		   (insert (format "%S " direction))))
 	    lm-directions)
@@ -1252,22 +1260,32 @@ because it is overwritten by \"One moment please\".")
   (mapc 'lm-print-wts-int lm-directions))
 
 ;;;_  - learning parameters
-(defvar lm-bound 0.005
-  "The maximum that w0j may be.")
-(defvar lm-c 1.0
+(defcustom lm-bound 0.005
+  "The maximum that w0j may be."
+  :type 'number
+  :group 'lm)
+(defcustom lm-c 1.0
   "A factor applied to modulate the increase in wij.
-Used in the function lm-update-normal-weights.")
-(defvar lm-c-naught 0.5
+Used in the function lm-update-normal-weights."
+  :type 'number
+  :group 'lm)
+(defcustom lm-c-naught 0.5
   "A factor applied to modulate the increase in w0j.
-Used in the function lm-update-naught-weights.")
+Used in the function lm-update-naught-weights."
+  :type 'number
+  :group 'lm)
 (defvar lm-initial-w0 0.0)
 (defvar lm-initial-wij 0.0)
-(defvar lm-no-payoff 0
+(defcustom lm-no-payoff 0
   "The amount of simulation cycles that have occurred with no movement.
-Used to move the robot when he is stuck in a rut for some reason.")
-(defvar lm-max-stall-time 2
+Used to move the robot when he is stuck in a rut for some reason."
+  :type 'integer
+  :group 'lm)
+(defcustom lm-max-stall-time 2
   "The maximum number of cycles that the robot can remain stuck in a place.
-After this limit is reached, lm-random-move is called to push him out of it.")
+After this limit is reached, lm-random-move is called to push him out of it."
+  :type 'integer
+  :group 'lm)
 
 
 ;;;_ + Randomizing functions
@@ -1282,7 +1300,7 @@ After this limit is reached, lm-random-move is called to push him out of it.")
 ;   (* (/ (random 900000) 900000.0) .0001)))
 ;;;_   : lm-randomize-weights-for (direction)
 (defun lm-randomize-weights-for (direction)
-  (mapc '(lambda (target-direction)
+  (mapc (lambda (target-direction)
 	     (put direction
 		  target-direction
 		  (* (lm-flip-a-coin) (/  (random 10000) 10000.0))))
@@ -1293,7 +1311,7 @@ After this limit is reached, lm-random-move is called to push him out of it.")
 
 ;;;_   : lm-fix-weights-for (direction)
 (defun lm-fix-weights-for (direction)
-  (mapc '(lambda (target-direction)
+  (mapc (lambda (target-direction)
 	     (put direction
 		  target-direction
 		  lm-initial-wij))
@@ -1377,7 +1395,7 @@ After this limit is reached, lm-random-move is called to push him out of it.")
 	   0.0))))
 
 (defun lm-update-normal-weights (direction)
-  (mapc '(lambda (target-direction)
+  (mapc (lambda (target-direction)
 	     (put direction target-direction
 		  (+
 		   (get direction target-direction)
@@ -1388,7 +1406,7 @@ After this limit is reached, lm-random-move is called to push him out of it.")
 	  lm-directions))
 
 (defun lm-update-naught-weights (direction)
-  (mapc '(lambda (target-direction)
+  (mapc (lambda (target-direction)
 	     (put direction 'w0
 		  (lm-f
 		   (+
@@ -1402,7 +1420,7 @@ After this limit is reached, lm-random-move is called to push him out of it.")
 ;;;_ + Statistics gathering and creating functions
 
 (defun lm-calc-current-smells ()
-  (mapc '(lambda (direction)
+  (mapc (lambda (direction)
 	     (put direction 'smell (calc-smell-internal direction)))
 	  lm-directions))
 
@@ -1414,7 +1432,7 @@ After this limit is reached, lm-random-move is called to push him out of it.")
     (setf lm-no-payoff 0)))
 
 (defun lm-store-old-y_t ()
-  (mapc '(lambda (direction)
+  (mapc (lambda (direction)
 	     (put direction 'y_t-1 (get direction 'y_t)))
 	  lm-directions))
 
@@ -1422,35 +1440,33 @@ After this limit is reached, lm-random-move is called to push him out of it.")
 ;;;_ + Functions to move robot
 
 (defun lm-confidence-for (target-direction)
-  (+
-   (get target-direction 'w0)
-   (reduce '+
-    (mapcar '(lambda (direction)
-	       (*
-		(get direction target-direction)
-		(get direction 'smell))
-			)
-	    lm-directions))))
+  (apply '+
+	 (get target-direction 'w0)
+	 (mapcar (lambda (direction)
+		   (*
+		    (get direction target-direction)
+		    (get direction 'smell)))
+		 lm-directions)))
 
 
 (defun lm-calc-confidences ()
-  (mapc '(lambda (direction)
+  (mapc (lambda (direction)
 	     (put direction 's (lm-confidence-for direction)))
 	     lm-directions))
 
 (defun lm-move ()
   (if (and (= (get 'lm-n 'y_t) 1.0) (= (get 'lm-s 'y_t) 1.0))
       (progn
-	(mapc '(lambda (dir) (put dir 'y_t 0)) lm-ns)
+	(mapc (lambda (dir) (put dir 'y_t 0)) lm-ns)
 	(if lm-debug
 	    (message "n-s normalization."))))
   (if (and (= (get 'lm-w 'y_t) 1.0) (= (get 'lm-e 'y_t) 1.0))
       (progn
-	(mapc '(lambda (dir) (put dir 'y_t 0)) lm-ew)
+	(mapc (lambda (dir) (put dir 'y_t 0)) lm-ew)
 	(if lm-debug
 	    (message "e-w normalization"))))
 
-  (mapc '(lambda (pair)
+  (mapc (lambda (pair)
 	     (if (> (get (car pair) 'y_t) 0)
 		 (funcall (car (cdr pair)))))
 	  '(
@@ -1466,7 +1482,7 @@ After this limit is reached, lm-random-move is called to push him out of it.")
 
 (defun lm-random-move ()
   (mapc
-   '(lambda (direction) (put direction 'y_t 0))
+   (lambda (direction) (put direction 'y_t 0))
    lm-directions)
   (dolist (direction (nth (random 8) lm-8-directions))
     (put direction 'y_t 1.0))
@@ -1562,14 +1578,14 @@ If the game is finished, this command requests for another game."
 
   (lm-set-landmark-signal-strengths)
 
-  (mapc '(lambda (direction)
+  (mapc (lambda (direction)
 	     (put direction 'y_t 0.0))
 	  lm-directions)
 
   (if (not save-weights)
       (progn
 	(mapc 'lm-fix-weights-for lm-directions)
-	(mapc '(lambda (direction)
+	(mapc (lambda (direction)
 		   (put direction 'w0 lm-initial-w0))
 	lm-directions))
     (message "Weights preserved for this run."))
@@ -1599,10 +1615,10 @@ If the game is finished, this command requests for another game."
 
   (setq lm-tree-r       (* (sqrt (+ (square lm-cx) (square lm-cy))) 1.5))
 
-  (mapc '(lambda (direction)
+  (mapc (lambda (direction)
 	     (put direction 'r (* lm-cx 1.1)))
 	lm-ew)
-  (mapc '(lambda (direction)
+  (mapc (lambda (direction)
 	     (put direction 'r (* lm-cy 1.1)))
 	lm-ns)
   (put 'lm-tree 'r lm-tree-r))
@@ -1691,4 +1707,6 @@ Use \\[describe-mode] for more info."
 ;;;outline-layout: (0 : -1 -1 0)
 ;;;End:
 
+(provide 'landmark)
+
 ;;; landmark.el ends here
