@@ -1,4 +1,4 @@
-;;; sort.el --- commands to sort text in an Emacs buffer.
+;;; sort.el --- commands to sort text in an Emacs buffer
 
 ;; Copyright (C) 1986, 1987, 1994, 1995 Free Software Foundation, Inc.
 
@@ -30,8 +30,14 @@
 
 ;;; Code:
 
-(defvar sort-fold-case nil
-  "*Non-nil if the buffer sort functions should ignore case.")
+(defgroup sort nil
+  "Commands to sort text in an Emacs buffer."
+  :group 'data)
+
+(defcustom sort-fold-case nil
+  "*Non-nil if the buffer sort functions should ignore case."
+  :group 'sort
+  :type 'boolean)
 
 ;;;###autoload
 (defun sort-subr (reverse nextrecfun endrecfun &optional startkeyfun endkeyfun)
@@ -46,6 +52,8 @@ contiguous.
 
 Usually the records are rearranged in order of ascending sort key.
 If REVERSE is non-nil, they are rearranged in order of descending sort key.
+The variable `sort-fold-case' determines whether alphabetic case affects
+the sort order.
 
 The next four arguments are functions to be called to move point
 across a sort record.  They will be called many times from within sort-subr.
@@ -192,7 +200,9 @@ same as ENDRECFUN."
 (defun sort-lines (reverse beg end) 
   "Sort lines in region alphabetically; argument means descending order.
 Called from a program, there are three arguments:
-REVERSE (non-nil means reverse order), BEG and END (region to sort)."
+REVERSE (non-nil means reverse order), BEG and END (region to sort).
+The variable `sort-fold-case' determines whether alphabetic case affects
+the sort order."
   (interactive "P\nr")
   (save-excursion
     (save-restriction
@@ -204,7 +214,9 @@ REVERSE (non-nil means reverse order), BEG and END (region to sort)."
 (defun sort-paragraphs (reverse beg end)
   "Sort paragraphs in region alphabetically; argument means descending order.
 Called from a program, there are three arguments:
-REVERSE (non-nil means reverse order), BEG and END (region to sort)."
+REVERSE (non-nil means reverse order), BEG and END (region to sort).
+The variable `sort-fold-case' determines whether alphabetic case affects
+the sort order."
   (interactive "P\nr")
   (save-excursion
     (save-restriction
@@ -221,7 +233,9 @@ REVERSE (non-nil means reverse order), BEG and END (region to sort)."
 (defun sort-pages (reverse beg end)
   "Sort pages in region alphabetically; argument means descending order.
 Called from a program, there are three arguments:
-REVERSE (non-nil means reverse order), BEG and END (region to sort)."
+REVERSE (non-nil means reverse order), BEG and END (region to sort).
+The variable `sort-fold-case' determines whether alphabetic case affects
+the sort order."
   (interactive "P\nr")
   (save-excursion
     (save-restriction
@@ -244,26 +258,40 @@ REVERSE (non-nil means reverse order), BEG and END (region to sort)."
     (modify-syntax-entry ?\. "_" table)	; for floating pt. numbers. -wsr
     (setq sort-fields-syntax-table table)))
 
+(defcustom sort-numeric-base 10
+  "*The default base used by `sort-numeric-fields'."
+  :group 'sort
+  :type 'integer)
+
 ;;;###autoload
 (defun sort-numeric-fields (field beg end)
   "Sort lines in region numerically by the ARGth field of each line.
 Fields are separated by whitespace and numbered from 1 up.
-Specified field must contain a number in each line of the region.
+Specified field must contain a number in each line of the region,
+which may begin with \"0x\" or \"0\" for hexadecimal and octal values.
+Otherwise, the number is interpreted according to sort-numeric-base.
 With a negative arg, sorts by the ARGth field counted from the right.
 Called from a program, there are three arguments:
 FIELD, BEG and END.  BEG and END specify region to sort."
   (interactive "p\nr")
   (sort-fields-1 field beg end
-		 (function (lambda ()
-			     (sort-skip-fields field)
-			     (string-to-number
-			      (buffer-substring
-			        (point)
-				(save-excursion
-				  ;; This is just wrong! Even without floats...
-				  ;; (skip-chars-forward "[0-9]")
-				  (forward-sexp 1)
-				  (point))))))
+		 (lambda ()
+		   (sort-skip-fields field)
+		   (let* ((case-fold-search t)
+			  (base
+			   (if (looking-at "\\(0x\\)[0-9a-f]\\|\\(0\\)[0-7]")
+			       (cond ((match-beginning 1)
+				      (goto-char (match-end 1))
+				      16)
+				     ((match-beginning 2)
+				      (goto-char (match-end 2))
+				      8)
+				     (t nil)))))
+		     (string-to-number (buffer-substring (point)
+							 (save-excursion
+							   (forward-sexp 1)
+							   (point)))
+				       (or base sort-numeric-base))))
 		 nil))
 
 ;;;;;###autoload
@@ -293,7 +321,9 @@ FIELD, BEG and END.  BEG and END specify region to sort."
 Fields are separated by whitespace and numbered from 1 up.
 With a negative arg, sorts by the ARGth field counted from the right.
 Called from a program, there are three arguments:
-FIELD, BEG and END.  BEG and END specify region to sort."
+FIELD, BEG and END.  BEG and END specify region to sort.
+The variable `sort-fold-case' determines whether alphabetic case affects
+the sort order."
   (interactive "p\nr")
   (sort-fields-1 field beg end
 		 (function (lambda ()
@@ -382,6 +412,9 @@ If a match for KEY is not found within a record then that record is ignored.
 
 With a negative prefix arg sorts in reverse order.
 
+The variable `sort-fold-case' determines whether alphabetic case affects
+the sort order.
+
 For example: to sort lines in the region by the first word on each line
  starting with the letter \"f\",
  RECORD-REGEXP would be \"^.*$\" and KEY would be \"\\\\=\\<f\\\\w*\\\\>\""
@@ -416,11 +449,8 @@ sRegexp specifying key within record: \nr")
 					(setq n 0))
 				       (t (throw 'key nil)))
 				 (condition-case ()
-				     (if (fboundp 'buffer-substring-lessp)
-					 (cons (match-beginning n)
-					       (match-end n))
-					 (buffer-substring (match-beginning n)
-							   (match-end n)))
+				     (cons (match-beginning n)
+					   (match-end n))
 				   ;; if there was no such register
 				   (error (throw 'key nil)))))))))))
 
@@ -430,10 +460,12 @@ sRegexp specifying key within record: \nr")
 ;;;###autoload
 (defun sort-columns (reverse &optional beg end)
   "Sort lines in region alphabetically by a certain range of columns.
-For the purpose of this command, the region includes
+For the purpose of this command, the region BEG...END includes
 the entire line that point is in and the entire line the mark is in.
 The column positions of point and mark bound the range of columns to sort on.
-A prefix argument means sort into reverse order.
+A prefix argument means sort into REVERSE order.
+The variable `sort-fold-case' determines whether alphabetic case affects
+the sort order.
 
 Note that `sort-columns' rejects text that contains tabs,
 because tabs could be split across the specified columns
@@ -454,21 +486,27 @@ Use \\[untabify] to convert tabs to spaces before sorting."
       (setq col-start (min col-beg1 col-end1))
       (setq col-end (max col-beg1 col-end1))
       (if (search-backward "\t" beg1 t)
-	  (error "sort-columns does not work with tabs.  Use M-x untabify."))
-      (if (not (eq system-type 'vax-vms))
+	  (error "sort-columns does not work with tabs -- use M-x untabify"))
+      (if (not (or (eq system-type 'vax-vms)
+		   (text-properties-at beg1)
+		   (< (next-property-change beg1 nil end1) end1)))
 	  ;; Use the sort utility if we can; it is 4 times as fast.
-	  (call-process-region beg1 end1 "sort" t t nil
-			       (if reverse "-rt\n" "-t\n")
-			       (concat "+0." col-start)
-			       (concat "-0." col-end))
+	  ;; Do not use it if there are any properties in the region,
+	  ;; since the sort utility would lose the properties.
+	  (let ((sort-args (list (if reverse "-rt\n" "-t\n")
+				 (concat "+0." (int-to-string col-start))
+				 (concat "-0." (int-to-string col-end)))))
+	    (when sort-fold-case
+	      (push "-f" sort-args))
+	    (apply #'call-process-region beg1 end1 "sort" t t nil sort-args))
 	;; On VMS, use Emacs's own facilities.
 	(save-excursion
 	  (save-restriction
 	    (narrow-to-region beg1 end1)
 	    (goto-char beg1)
 	    (sort-subr reverse 'forward-line 'end-of-line
-		       (function (lambda () (move-to-column col-start) nil))
-		       (function (lambda () (move-to-column col-end) nil)))))))))
+		       #'(lambda () (move-to-column col-start) nil)
+		       #'(lambda () (move-to-column col-end) nil))))))))
 
 ;;;###autoload
 (defun reverse-region (beg end)
