@@ -222,13 +222,13 @@ Returns the abbrev symbol, if expansion took place.")
 {
   register char *buffer, *p;
   int wordstart, wordend;
-  register int wordstart_byte, wordend_byte, idx;
+  register int wordstart_byte, wordend_byte, idx, idx_byte;
   int whitecnt;
   int uccount = 0, lccount = 0;
   register Lisp_Object sym;
   Lisp_Object expansion, hook, tem;
-  int oldmodiff = MODIFF;
   Lisp_Object value;
+  int multibyte = ! NILP (current_buffer->enable_multibyte_characters);
 
   value = Qnil;
 
@@ -275,25 +275,38 @@ Returns the abbrev symbol, if expansion took place.")
 
   p = buffer = (char *) alloca (wordend_byte - wordstart_byte);
 
-  for (idx = wordstart_byte; idx < wordend_byte; idx++)
+  for (idx = wordstart, idx_byte = wordstart_byte; idx < wordend;)
     {
-      /* ??? This loop needs to go by characters!  */
-      register int c = FETCH_BYTE (idx);
+      register int c;
+
+      if (multibyte)
+	{
+	  FETCH_CHAR_ADVANCE (c, idx, idx_byte);
+	}
+      else
+	{
+	  c = FETCH_BYTE (idx_byte);
+	  idx++, idx_byte++;
+	}
+
       if (UPPERCASEP (c))
 	c = DOWNCASE (c), uccount++;
       else if (! NOCASEP (c))
 	lccount++;
-      *p++ = c;
+      if (multibyte)
+	p += CHAR_STRING (c, p);
+      else
+	*p++ = c;
     }
 
   if (VECTORP (current_buffer->abbrev_table))
     sym = oblookup (current_buffer->abbrev_table, buffer,
-		    wordend - wordstart, wordend_byte - wordstart_byte);
+		    wordend - wordstart, p - buffer);
   else
     XSETFASTINT (sym, 0);
   if (INTEGERP (sym) || NILP (XSYMBOL (sym)->value))
     sym = oblookup (Vglobal_abbrev_table, buffer,
-		    wordend - wordstart, wordend_byte - wordstart_byte);
+		    wordend - wordstart, p - buffer);
   if (INTEGERP (sym) || NILP (XSYMBOL (sym)->value))
     return value;
 
@@ -365,7 +378,23 @@ Returns the abbrev symbol, if expansion took place.")
 
   hook = XSYMBOL (sym)->function;
   if (!NILP (hook))
-    call0 (hook);
+    {
+      Lisp_Object expanded, prop;
+
+      /* If the abbrev has a hook function, run it.  */
+      expanded = call0 (hook);
+
+      /* In addition, if the hook function is a symbol with a a
+	 non-nil `no-self-insert' property, let the value it returned
+	 specify whether we consider that an expansion took place.  If
+	 it returns nil, no expansion has been done.  */
+
+      if (SYMBOLP (hook)
+	  && NILP (expanded)
+	  && (prop = Fget (hook, intern ("no-self-insert")),
+	      !NILP (prop)))
+	value = Qnil;
+    }
 
   return value;
 }
@@ -540,6 +569,7 @@ for any particular abbrev defined in both.");
     "The abbrev table of mode-specific abbrevs for Fundamental Mode.");
   Vfundamental_mode_abbrev_table = Fmake_abbrev_table ();
   current_buffer->abbrev_table = Vfundamental_mode_abbrev_table;
+  buffer_defaults.abbrev_table = Vfundamental_mode_abbrev_table;
 
   DEFVAR_LISP ("last-abbrev", &Vlast_abbrev,
     "The abbrev-symbol of the last abbrev expanded.  See `abbrev-symbol'.");
