@@ -1,5 +1,6 @@
 /* Call a Lisp function interactively.
-   Copyright (C) 1985, 1986, 1993, 1994, 1995 Free Software Foundation, Inc.
+   Copyright (C) 1985, 86, 93, 94, 95, 1997, 2000
+   Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -27,7 +28,9 @@ Boston, MA 02111-1307, USA.  */
 #include "window.h"
 #include "mocklisp.h"
 
-extern char *index ();
+#ifdef HAVE_INDEX
+extern char *index P_ ((const char *, int));
+#endif
 
 extern Lisp_Object Qcursor_in_echo_area;
 
@@ -35,6 +38,8 @@ Lisp_Object Vcurrent_prefix_arg, Qminus, Qplus;
 Lisp_Object Qcall_interactively;
 Lisp_Object Vcommand_history;
 
+extern Lisp_Object Vhistory_length;
+
 Lisp_Object Vcommand_debug_status, Qcommand_debug_status;
 Lisp_Object Qenable_recursive_minibuffers;
 
@@ -83,7 +88,7 @@ Just `(interactive)' means pass no args when calling interactively.\n\
 a -- Function name: symbol with a function definition.\n\
 b -- Name of existing buffer.\n\
 B -- Name of buffer, possibly nonexistent.\n\
-c -- Character.\n\
+c -- Character (no input method is used).\n\
 C -- Command name: symbol with interactive function definition.\n\
 d -- Value of point as number.  Does not do I/O.\n\
 D -- Directory name.\n\
@@ -146,11 +151,11 @@ quotify_args (exp)
      Lisp_Object exp;
 {
   register Lisp_Object tail;
-  register struct Lisp_Cons *ptr;
-  for (tail = exp; CONSP (tail); tail = ptr->cdr)
+  Lisp_Object next;
+  for (tail = exp; CONSP (tail); tail = next)
     {
-      ptr = XCONS (tail);
-      ptr->car = quotify_arg (ptr->car);
+      next = XCDR (tail);
+      XCAR (tail) = quotify_arg (XCAR (tail));
     }
   return exp;
 }
@@ -181,7 +186,9 @@ See `interactive'.\n\
 \n\
 Optional second arg RECORD-FLAG non-nil\n\
 means unconditionally put this command in the command-history.\n\
-Otherwise, this is done only if an arg is read using the minibuffer.")
+Otherwise, this is done only if an arg is read using the minibuffer.\n\
+Optional third arg KEYS, if given, specifies the sequence of events to\n\
+supply if the command inquires which events were used to invoke it.")
   (function, record_flag, keys)
      Lisp_Object function, record_flag, keys;
 {
@@ -230,6 +237,8 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 
   if (SYMBOLP (function))
     enable = Fget (function, Qenable_recursive_minibuffers);
+  else
+    enable = Qnil;
 
   fun = indirect_function (function);
 
@@ -288,8 +297,9 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
     {
       /* Make a copy of string so that if a GC relocates specs,
 	 `string' will still be valid.  */
-      string = (unsigned char *) alloca (XSTRING (specs)->size + 1);
-      bcopy (XSTRING (specs)->data, string, XSTRING (specs)->size + 1);
+      string = (unsigned char *) alloca (STRING_BYTES (XSTRING (specs)) + 1);
+      bcopy (XSTRING (specs)->data, string,
+	     STRING_BYTES (XSTRING (specs)) + 1);
     }
   else if (string == 0)
     {
@@ -311,17 +321,17 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	     instead of the present values.  */
 	  if (CONSP (input))
 	    {
-	      car = XCONS (input)->car;
+	      car = XCAR (input);
 	      /* Skip through certain special forms.  */
 	      while (EQ (car, Qlet) || EQ (car, Qletx)
 		     || EQ (car, Qsave_excursion))
 		{
-		  while (CONSP (XCONS (input)->cdr))
-		    input = XCONS (input)->cdr;
-		  input = XCONS (input)->car;
+		  while (CONSP (XCDR (input)))
+		    input = XCDR (input);
+		  input = XCAR (input);
 		  if (!CONSP (input))
 		    break;
-		  car = XCONS (input)->car;
+		  car = XCAR (input);
 		}
 	      if (EQ (car, Qlist))
 		{
@@ -344,6 +354,14 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	    }
 	  Vcommand_history
 	    = Fcons (Fcons (function, values), Vcommand_history);
+
+	  /* Don't keep command history around forever.  */
+	  if (NUMBERP (Vhistory_length) && XINT (Vhistory_length) > 0)
+	    {
+	      teml = Fnthcdr (Vhistory_length, Vcommand_history);
+	      if (CONSP (teml))
+		XCDR (teml) = Qnil;
+	    }
 	}
       single_kboard_state ();
       return apply1 (function, specs);
@@ -377,9 +395,9 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 
 	  event = XVECTOR (keys)->contents[next_event];
 	  if (EVENT_HAS_PARAMETERS (event)
-	      && (event = XCONS (event)->cdr, CONSP (event))
-	      && (event = XCONS (event)->car, CONSP (event))
-	      && (event = XCONS (event)->car, WINDOWP (event)))
+	      && (event = XCDR (event), CONSP (event))
+	      && (event = XCAR (event), CONSP (event))
+	      && (event = XCAR (event), WINDOWP (event)))
 	    {
 	      if (MINI_WINDOW_P (XWINDOW (event))
 		  && ! (minibuf_level > 0 && EQ (event, minibuf_window)))
@@ -436,15 +454,15 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
     {
       strncpy (prompt1, tem + 1, sizeof prompt1 - 1);
       prompt1[sizeof prompt1 - 1] = 0;
-      tem1 = index (prompt1, '\n');
+      tem1 = (char *) index (prompt1, '\n');
       if (tem1) *tem1 = 0;
       /* Fill argstrings with a vector of C strings
 	 corresponding to the Lisp strings in visargs.  */
       for (j = 1; j < i; j++)
 	argstrings[j]
-	  = EQ (visargs[j], Qnil)
-	    ? (unsigned char *) ""
-	      : XSTRING (visargs[j])->data;
+	  = (EQ (visargs[j], Qnil)
+	     ? (unsigned char *) ""
+	     : XSTRING (visargs[j])->data);
 
       /* Process the format-string in prompt1, putting the output
 	 into callint_message.  Make callint_message bigger if necessary.
@@ -454,7 +472,7 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	{
 	  int nchars = doprnt (callint_message, callint_message_size,
 			       prompt1, (char *)0,
-			       j - 1, argstrings + 1);
+			       j - 1, (char **) argstrings + 1);
 	  if (nchars < callint_message_size)
 	    break;
 	  callint_message_size *= 2;
@@ -476,22 +494,18 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	case 'b':   		/* Name of existing buffer */
 	  args[i] = Fcurrent_buffer ();
 	  if (EQ (selected_window, minibuf_window))
-	    args[i] = Fother_buffer (args[i], Qnil);
+	    args[i] = Fother_buffer (args[i], Qnil, Qnil);
 	  args[i] = Fread_buffer (build_string (callint_message), args[i], Qt);
 	  break;
 
 	case 'B':		/* Name of buffer, possibly nonexistent */
 	  args[i] = Fread_buffer (build_string (callint_message),
-				  Fother_buffer (Fcurrent_buffer (), Qnil),
+				  Fother_buffer (Fcurrent_buffer (), Qnil, Qnil),
 				  Qnil);
 	  break;
 
         case 'c':		/* Character */
-	  /* Use message_nolog rather than message1_nolog here,
-	     so that nothing bad happens if callint_message is changed
-	     within Fread_char (by a timer, for example).  */
-	  message_nolog ("%s", callint_message);
-	  args[i] = Fread_char ();
+	  args[i] = Fread_char (build_string (callint_message), Qnil);
 	  message1_nolog ((char *) 0);
 	  /* Passing args[i] directly stimulates compiler bug */
 	  teml = args[i];
@@ -508,7 +522,7 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	  break;
 
 	case 'd':		/* Value of point.  Does not do I/O.  */
-	  Fset_marker (point_marker, make_number (PT), Qnil);
+	  set_marker_both (point_marker, Qnil, PT, PT_BYTE);
 	  args[i] = point_marker;
 	  /* visargs[i] = Qnil; */
 	  varies[i] = 1;
@@ -538,10 +552,26 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	    int speccount1 = specpdl_ptr - specpdl;
 	    specbind (Qcursor_in_echo_area, Qt);
 	    args[i] = Fread_key_sequence (build_string (callint_message),
-					  Qnil, Qnil, Qnil);
+					  Qnil, Qnil, Qnil, Qnil);
 	    unbind_to (speccount1, Qnil);
 	    teml = args[i];
 	    visargs[i] = Fkey_description (teml);
+
+	    /* If the key sequence ends with a down-event,
+	       discard the following up-event.  */
+	    teml = Faref (args[i], make_number (XINT (Flength (args[i])) - 1));
+	    if (CONSP (teml))
+	      teml = XCAR (teml);
+	    if (SYMBOLP (teml))
+	      {
+		Lisp_Object tem2;
+
+		teml = Fget (teml, intern ("event-symbol-elements"));
+		/* Ignore first element, which is the base key.  */
+		tem2 = Fmemq (intern ("down"), Fcdr (teml));
+		if (! NILP (tem2))
+		  Fread_event (Qnil, Qnil);
+	      }
 	  }
 	  break;
 
@@ -550,10 +580,26 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	    int speccount1 = specpdl_ptr - specpdl;
 	    specbind (Qcursor_in_echo_area, Qt);
 	    args[i] = Fread_key_sequence (build_string (callint_message),
-					  Qnil, Qt, Qnil);
+					  Qnil, Qt, Qnil, Qnil);
 	    teml = args[i];
 	    visargs[i] = Fkey_description (teml);
 	    unbind_to (speccount1, Qnil);
+
+	    /* If the key sequence ends with a down-event,
+	       discard the following up-event.  */
+	    teml = Faref (args[i], make_number (XINT (Flength (args[i])) - 1));
+	    if (CONSP (teml))
+	      teml = XCAR (teml);
+	    if (SYMBOLP (teml))
+	      {
+		Lisp_Object tem2;
+
+		teml = Fget (teml, intern ("event-symbol-elements"));
+		/* Ignore first element, which is the base key.  */
+		tem2 = Fmemq (intern ("down"), Fcdr (teml));
+		if (! NILP (tem2))
+		  Fread_event (Qnil, Qnil);
+	      }
 	  }
 	  break;
 
@@ -631,7 +677,7 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 
 	case 'r':		/* Region, point and mark as 2 args. */
 	  check_mark ();
-	  Fset_marker (point_marker, make_number (PT), Qnil);
+	  set_marker_both (point_marker, Qnil, PT, PT_BYTE);
 	  /* visargs[i+1] = Qnil; */
 	  foo = marker_position (current_buffer->mark);
 	  /* visargs[i] = Qnil; */
@@ -727,6 +773,13 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	}
       Vcommand_history = Fcons (Flist (count + 1, visargs),
 				Vcommand_history);
+      /* Don't keep command history around forever.  */
+      if (NUMBERP (Vhistory_length) && XINT (Vhistory_length) > 0)
+	{
+	  teml = Fnthcdr (Vhistory_length, Vcommand_history);
+	  if (CONSP (teml))
+	    XCDR (teml) = Qnil;
+	}
     }
 
   /* If we used a marker to hold point, mark, or an end of the region,
@@ -761,8 +814,8 @@ Its numeric meaning is what you would get from `(interactive \"p\")'.")
     XSETFASTINT (val, 1);
   else if (EQ (raw, Qminus))
     XSETINT (val, -1);
-  else if (CONSP (raw) && INTEGERP (XCONS (raw)->car))
-    XSETINT (val, XINT (XCONS (raw)->car));
+  else if (CONSP (raw) && INTEGERP (XCAR (raw)))
+    XSETINT (val, XINT (XCAR (raw)));
   else if (INTEGERP (raw))
     val = raw;
   else
@@ -771,6 +824,7 @@ Its numeric meaning is what you would get from `(interactive \"p\")'.")
   return val;
 }
 
+void
 syms_of_callint ()
 {
   point_marker = Fmake_marker ();
@@ -824,6 +878,10 @@ since it has been set to nil by the time you can look.\n\
 Instead, you should use the variable `current-prefix-arg', although\n\
 normally commands can get this prefix argument with (interactive \"P\").");
 
+  DEFVAR_KBOARD ("last-prefix-arg", Vlast_prefix_arg,
+    "The value of the prefix argument for the previous editing command.\n\
+See `prefix-arg' for the meaning of the value.");
+
   DEFVAR_LISP ("current-prefix-arg", &Vcurrent_prefix_arg,
     "The value of the prefix argument for this editing command.\n\
 It may be a number, or the symbol `-' for just a minus sign as arg,\n\
