@@ -101,19 +101,6 @@ struct cm
 extern struct cm Wcm;		/* Terminal capabilities */
 extern char PC;			/* Pad character */
 
-#if defined (HAVE_LIBNCURSES) && ! defined (NCURSES_OSPEED_T)
-extern short ospeed;
-#else
-#if defined (HAVE_TERMIOS_H) && defined (LINUX)
-#include <termios.h>
-/* HJL's version of libc is said to need this on the Alpha.
-   On the other hand, DEC OSF1 on the Alpha needs ospeed to be a short.  */
-extern speed_t ospeed;
-#else
-extern short ospeed;		/* Output speed (from sg_ospeed) */
-#endif
-#endif
-
 /* Shorthand */
 #ifndef NoCMShortHand
 #define curY		Wcm.cm_curY
