@@ -1,5 +1,5 @@
 /* m- file for Mips machines.
-   Copyright (C) 1987, 1992 Free Software Foundation, Inc.
+   Copyright (C) 1987, 1992, 1999 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -31,7 +31,9 @@ NOTE-END  */
 /* Define WORDS_BIG_ENDIAN iff lowest-numbered byte in a word
    is the most significant byte.  */
 
+#if ! (defined (__MIPSEL__) || defined (MIPSEL) || defined (_MIPSEL))
 #define WORDS_BIG_ENDIAN
+#endif
 
 /* Define NO_ARG_ARRAY if you cannot take the address of the first of a
  * group of arguments and treat it as an array of the arguments.  */
@@ -79,8 +81,10 @@ NOTE-END  */
 
 /* CDC EP/IX 1.4.3 uses /unix */
 
+#ifndef __linux__
 #undef KERNEL_FILE
 #define KERNEL_FILE "/unix"
+#endif /* not __linux__ */
 
 /* Define CANNOT_DUMP on machines where unexec does not work.
    Then the function dump-emacs will not be defined
@@ -120,17 +124,26 @@ NOTE-END  */
 /* This machine requires completely different unexec code
    which lives in a separate file.  Specify the file name.  */
 
+#ifndef __linux__
+#undef UNEXEC
 #define UNEXEC unexmips.o
+#endif /* not __linux__ */
 
 /* Describe layout of the address space in an executing process.  */
 
+#ifdef __linux__
+#define TEXT_START      0x00400000
+#define DATA_START      0x10000000
+#define DATA_SEG_BITS	0x10000000
+#else /* not __linux__ */
 #define TEXT_START 0x400000
 #define DATA_START 0x800000
+#endif /* __linux__ */
 
 /* Alter some of the options used when linking.  */
 
-#ifndef NEWSOS5
-#ifdef BSD
+#if !defined(NEWSOS5) && !defined(__linux__)
+#ifdef BSD_SYSTEM
 
 /* DECstations don't have this library.
    #define LIBS_MACHINE -lmld  */
@@ -138,11 +151,18 @@ NOTE-END  */
 #define LD_SWITCH_MACHINE -D 800000
 #define LIBS_DEBUG
 
+#if defined (__NetBSD__) || defined (__OpenBSD__)
+#else  /* bsd with elf */
 #define LINKER /bsd43/bin/ld
-  
-#else /* not BSD */
+#endif /* bsd with elf */
+#else /* not BSD_SYSTEM */
 
+#if defined(__GNUC__) && defined(_ABIN32)
+#define LIBS_MACHINE
+#else
 #define LIBS_MACHINE -lmld
+#endif
+
 #define LD_SWITCH_MACHINE -D 800000 -g3
 #define START_FILES pre-crt0.o /usr/lib/crt1.o
 #define LIB_STANDARD -lbsd -lc /usr/lib/crtn.o
@@ -155,8 +175,8 @@ NOTE-END  */
 #define HAVE_VFORK		/* Graciously provided by libX.a */
 #endif
 
-#endif /* not BSD */
-#endif /* not NEWSOS5 */
+#endif /* not BSD_SYSTEM */
+#endif /* not NEWSOS5 && not __linux__ */
 
 /* The standard definitions of these macros would work ok,
    but these are faster because the constants are short.  */
@@ -173,7 +193,7 @@ NOTE-END  */
    (((unsigned)(a) << (BITS_PER_INT-GCTYPEBITS-VALBITS))		\
     >> (BITS_PER_INT-GCTYPEBITS-VALBITS)))
 
-#ifndef NEWSOS5
+#if !defined (NEWSOS5) && !defined (__linux__)
 #ifdef USG
 
 /* Cancel certain parts of standard sysV support.  */
@@ -182,7 +202,7 @@ NOTE-END  */
 #undef static
 
 /* Don't try to use SIGIO or FIONREAD even though they are defined.  */
-#undef SIGIO
+#define BROKEN_SIGIO
 #define BROKEN_FIONREAD
 
 /* Describe special kernel features.  */
@@ -196,9 +216,6 @@ NOTE-END  */
 /* The `select' in the system won't work for pipes, so don't use it.  */
 #undef HAVE_SELECT /* override configuration decision */
 
-#define HAVE_GETWD
-#define HAVE_GETTIMEOFDAY
-
 #define HAVE_PTYS
 #define HAVE_SOCKETS
 
@@ -209,11 +226,11 @@ NOTE-END  */
 
 #endif /* USG */
 
-#ifdef BSD
+#ifdef BSD_SYSTEM
 #define COFF
 #define TERMINFO
 #undef MAIL_USE_FLOCK  /* Someone should check this.  */
 #undef HAVE_UNION_WAIT
-#endif /* BSD */
+#endif /* BSD_SYSTEM */
 
-#endif /* not NEWSOS5 */
+#endif /* not NEWSOS5 && not __linux__ */
