@@ -15,7 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* CHANGE: [Eric H. Herrin II; eric@ms.uky.edu - 15 Sept 1988] 
  * Modified the sequent.h file for the Sequent Symmetry machine.   
@@ -59,9 +60,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #undef KERNEL_FILE
 #define KERNEL_FILE "/dynix"
 
-/* define HAVE_GETPAGESIZE for Symmetry */
-#define HAVE_GETPAGESIZE
-
 /* Avoids a compiler bug */
 
 #define TAHOE_REGISTER_BUG
