@@ -1,7 +1,29 @@
-/* s/ file for freebsd system.  */
+/* System description header for FreeBSD systems.
+   This file describes the parameters that system description files
+   should define or not.
+   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+   Free Software Foundation, Inc.
+
+This file is part of GNU Emacs.
+
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
+
+GNU Emacs is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* Get the correct __FreeBSD_version, even if this is before that was
    defined. */
+#ifndef __FreeBSD_version 
 #ifndef __FreeBSD__
 #define __FreeBSD_version 199401
 #elif __FreeBSD__ == 1
@@ -9,6 +31,7 @@
 #else
 #include <osreldate.h>
 #endif
+#endif /* !defined __FreeBSD_version */
 
 /* '__FreeBSD__' is defined by the preprocessor on FreeBSD-1.1 and up.
    Earlier versions do not have shared libraries, so inhibit them.
@@ -55,7 +78,7 @@
 
 #ifdef __ELF__
 
-#define LD_SWITCH_SYSTEM
+#define LD_SWITCH_SYSTEM_1
 #define START_FILES pre-crt0.o /usr/lib/crt1.o /usr/lib/crti.o /usr/lib/crtbegin.o
 #define UNEXEC unexelf.o
 #define LIB_STANDARD -lgcc -lc -lgcc /usr/lib/crtend.o /usr/lib/crtn.o
@@ -65,7 +88,7 @@
 #else /* not __ELF__ */
 
 #ifndef NO_SHARED_LIBS
-#define LD_SWITCH_SYSTEM -e start -dc
+#define LD_SWITCH_SYSTEM_1 -e start -dc -L/usr/local/lib
 #define HAVE_TEXT_START		/* No need to define `start_of_text'. */
 #if __FreeBSD_version >= 300002
 #define START_FILES pre-crt0.o /usr/lib/aout/crt0.o
@@ -84,7 +107,7 @@
 #else /* NO_SHARED_LIBS */
 #ifdef __FreeBSD__  /* shared libs are available, but the user prefers
                      not to use them.  */
-#define LD_SWITCH_SYSTEM -Bstatic
+#define LD_SWITCH_SYSTEM_1 -Bstatic -L/usr/local/lib
 #define A_TEXT_OFFSET(x) (sizeof (struct exec))
 #define A_TEXT_SEEK(hdr) (N_TXTOFF(hdr) + A_TEXT_OFFSET(hdr))
 #endif /* __FreeBSD__ */
@@ -92,9 +115,17 @@
 
 #endif /* not __ELF__ */
 
+/* Let `ld' find image libs and similar things in /usr/local/lib.  The
+   system compiler, GCC, has apparently been modified to not look
+   there, contrary to what a stock GCC would do.  */
+
+#define LD_SWITCH_SYSTEM LD_SWITCH_SYSTEM_1 -L/usr/local/lib
+
 #define HAVE_WAIT_HEADER
-#define HAVE_GETLOADAVG
-/*#define HAVE_GETPAGESIZE  /* configure now puts this in config.h */
+#define HAVE_GETLOADAVG 1
+#if 0
+#define HAVE_GETPAGESIZE  /* configure now puts this in config.h */
+#endif
 #define HAVE_TERMIOS
 #define NO_TERMIO
 #define DECLARE_GETPWUID_WITH_UID_T
@@ -113,12 +144,15 @@
 #define BSD_SYSTEM 199506
 #endif
 
+#if 0  /* Shouldn't be necessary and produces warnings with the 
+          experimental Autoconf test.  */
 #define WAITTYPE int
 /* get this since it won't be included if WAITTYPE is defined */
 #ifdef emacs
 #include <sys/wait.h>
 #endif
 #define WRETCODE(w) (_W_INT(w) >> 8)
+#endif
 
 /* Needed to avoid hanging when child process writes an error message
    and exits -- enami tsugutomo <enami@ba2.so-net.or.jp>.  */
@@ -135,6 +169,21 @@
    To avoid this problem, you could #undef it to use no file lock. */
 /* #undef CLASH_DETECTION */
 
+/* If the system's imake configuration file defines `NeedWidePrototypes'
+   as `NO', we must define NARROWPROTO manually.  Such a define is
+   generated in the Makefile generated by `xmkmf'.  If we don't 
+   define NARROWPROTO, we will see the wrong function prototypes
+   for X functions taking float or double parameters.  */
+
+#define NARROWPROTO 1
+
+/* The following is needed to make `configure' find Xpm, Xaw3d and
+   image include and library files if using /usr/bin/gcc.  That
+   compiler seems to be modified to not find headers in
+   /usr/local/include or libs in /usr/local/lib by default.  */
+
+#define C_SWITCH_SYSTEM -I/usr/X11R6/include -I/usr/local/include -L/usr/local/lib
+
 /* Circumvent a bug in FreeBSD.  In the following sequence of
    writes/reads on a PTY, read(2) returns bogus data:
 
@@ -150,3 +199,24 @@
    be seen again in a later read(2), without the CRs.  */
 
 #define BROKEN_PTY_READ_AFTER_EAGAIN 1
+
+/* Tell that garbage collector that setjmp is known to save all
+   registers relevant for conservative garbage collection in the
+   jmp_buf.  */
+
+#define GC_SETJMP_WORKS 1
+
+/* Use the GC_MAKE_GCPROS_NOOPS (see lisp.h) method for marking the
+   stack.  */
+
+#define GC_MARK_STACK 	GC_MAKE_GCPROS_NOOPS
+
+/* Define USE_MMAP_FOR_BUFFERS to let Emacs use mmap(2) to allocate
+   buffer text.  This overrides REL_ALLOC.  */
+
+#define USE_MMAP_FOR_BUFFERS	1
+
+/* Use sigprocmask(2) and friends instead of sigblock(2); the man page
+   of sigblock says it is obsolete.  */
+
+#define POSIX_SIGNALS		1
