@@ -1,6 +1,6 @@
 /* System description file for MS-DOS
 
-   Copyright (C) 1993, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1993, 1996, 1997, 2001 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -35,7 +35,7 @@ Boston, MA 02111-1307, USA.  */
 /* #define BSD4_1 */
 /* #define BSD4_2 */
 /* #define BSD4_3 */
-/* #define BSD */
+/* #define BSD_SYSTEM */
 /* #define VMS */
 #ifndef MSDOS
 #define MSDOS
@@ -50,7 +50,7 @@ You lose; /* Emacs for DOS must be compiled with DJGPP */
 #endif
 
 #define DOS_NT	/* MSDOS or WINDOWSNT */
-#undef BSD
+#undef BSD_SYSTEM
 #undef VMS
 
 /* SYSTEM_TYPE should indicate the kind of system you are using.
@@ -58,7 +58,7 @@ You lose; /* Emacs for DOS must be compiled with DJGPP */
 
 #define SYSTEM_TYPE "ms-dos"
 
-#define SYMS_SYSTEM syms_of_dosfns();syms_of_msdos()
+#define SYMS_SYSTEM syms_of_dosfns();syms_of_msdos();syms_of_win16select()
 
 /* NOMULTIPLEJOBS should be defined if your system's shell
  does not have "job control" (the ability to stop a program,
@@ -216,7 +216,9 @@ You lose; /* Emacs for DOS must be compiled with DJGPP */
 
 /* When $TERM is "internal" then this is substituted:  */
 #define INTERNAL_TERMINAL "pc|bios|IBM PC with colour display:\
-:co#80:li#25:km:ms:cm=<CM>:cl=<CL>:ce=<CE>:"
+:co#80:li#25:Co#16:pa#256:km:ms:cm=<CM>:cl=<CL>:ce=<CE>:\
+:se=</SO>:so=<SO>:us=<UL>:ue=</UL>:md=<BD>:mh=<DIM>:mb=<BL>:mr=<RV>:me=<NV>:\
+:AB=<BG %d>:AF=<FG %d>:op=<DefC>:"
 
 /* Define this to a function (Fdowncase, Fupcase) if your file system
    likes that */
@@ -265,5 +267,12 @@ You lose; /* Emacs for DOS must be compiled with DJGPP */
 #define LIBX11_SYSTEM -lxext -lsys
 #else
 /* We need a little extra space, see ../../lisp/loadup.el */
-#define SYSTEM_PURESIZE_EXTRA 85000
+#define SYSTEM_PURESIZE_EXTRA 60000
 #endif
+
+/* Tell the garbage collector that setjmp is known to save all
+   registers relevant for conservative garbage collection in the
+   jmp_buf.  */
+
+#define GC_SETJMP_WORKS 1
+#define GC_MARK_STACK GC_MAKE_GCPROS_NOOPS
