@@ -1,5 +1,5 @@
 /* Definitions file for GNU Emacs running SCO Xenix 386 Release 2.2
-   Copyright (C) 1988 Free Software Foundation, Inc.
+   Copyright (C) 1988, 1999 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -32,7 +32,7 @@ Boston, MA 02111-1307, USA.  */
 /* #define BSD4_1 */
 /* #define BSD4_2 */
 /* #define BSD4_3 */
-/* #define BSD */
+/* #define BSD_SYSTEM */
 /* #define VMS */
 
 /* SYSTEM_TYPE should indicate the kind of system you are using.
@@ -136,10 +136,6 @@ Boston, MA 02111-1307, USA.  */
 
 /* #define SHORTNAMES */
 
-/* We do not use the Berkeley (and usg5.2.2) interface to nlist.  */
-
-/* #define NLIST_STRUCT */
-
 /* Compensate for one incompatibility between Xenix and V.0.  */
 #define n_zeroes n_name[0]
 
@@ -168,20 +164,6 @@ Boston, MA 02111-1307, USA.  */
 #define _setjmp setjmp
 #define _longjmp longjmp
 
-/* On USG systems the system calls are interruptible by signals
- that the user program has elected to catch.  Thus the system call
- must be retried in these cases.  To handle this without massive
- changes in the source code, we remap the standard system call names
- to names for our own functions in sysdep.c that do the system call
- with retries. */
-
-#define read sys_read
-#define open sys_open
-#define write sys_write
-
-#define INTERRUPTIBLE_OPEN
-#define INTERRUPTIBLE_IO
-
 /* On USG systems these have different names */
 
 #define index strchr
