@@ -1,5 +1,5 @@
 /* Unexec for Sunos 4 using shared libraries.
-   Copyright (C) 1990, 1994 Free Software Foundation, Inc.
+   Copyright (C) 1990, 1994, 1999 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -15,7 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* Contributed by Viktor Dukhovni.  */
 /*
@@ -42,6 +43,11 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
  *  is somewhat abused here) is loaded first!
  *
  */
+
+#ifdef emacs
+#include <config.h>
+#endif
+
 #include <sys/param.h>
 #include <sys/mman.h>
 #include <sys/file.h>
@@ -50,13 +56,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <stdio.h>
 #include <a.out.h>
 
-/* Do this after the above #include's in case a configuration file wants
-   to define things for this file based on what <a.out.h> defines.  */
-#ifdef emacs
-#include <config.h>
-#endif
-
-#if defined (SUNOS4) || defined (__FreeBSD__)
+#if defined (SUNOS4) || defined (__FreeBSD__) || defined (__NetBSD__)
 #define UNDO_RELOCATION
 #endif
 
@@ -92,7 +92,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #endif
 
 extern char *getenv ();
-static unsigned Brk;
+static unsigned brk_value;
 static struct exec nhdr;
 static int rd_only_len;
 static long cookie;
@@ -107,10 +107,6 @@ unexec (new_name, a_name, bndry, bss_start, entry)
   struct exec ohdr;		/* Allocate on the stack,  not needed in the next life */
   struct stat stat;
 
-#ifdef emacs
-  fprintf (stderr, "Used %d bytes of Pure Storage\n", pureptr);
-#endif
-
   if ((fd = open (a_name, O_RDONLY)) < 0)
     {
       fprintf (stderr, "%s: open: ", a_name);
@@ -149,7 +145,8 @@ unexec (new_name, a_name, bndry, bss_start, entry)
    */
   cookie = time (0);
 
-  Brk = sbrk (0);		/* Save the break, it is reset to &_end (by ld.so?) */
+  /* Save the break, it is reset to &_end (by ld.so?).  */
+  brk_value = (unsigned) sbrk (0);
 
   /*
    * Round up data start to a page boundary (Lose if not a 2 power!)
@@ -167,15 +164,13 @@ unexec (new_name, a_name, bndry, bss_start, entry)
   initialized = 1;
 #endif
   
-  /* 
-   * Handle new data and bss sizes and optional new entry point.
-   * No one actually uses bss_start and entry,  but tradition compels
-   * one to support them.
-   * Could complain if bss_start > Brk,  but the caller is *supposed* to know
-   * what she is doing.
-   */
-  nhdr.a_data = (bss_start ? bss_start : Brk) - N_DATADDR (nhdr);
-  nhdr.a_bss  = bss_start ? Brk - bss_start : 0;
+  /* Handle new data and bss sizes and optional new entry point.
+     No one actually uses bss_start and entry,  but tradition compels
+     one to support them.
+     Could complain if bss_start > brk_value,
+     but the caller is *supposed* to know what she is doing.  */
+  nhdr.a_data = (bss_start ? bss_start : brk_value) - N_DATADDR (nhdr);
+  nhdr.a_bss  = bss_start ? brk_value - bss_start : 0;
   if (entry) 
     nhdr.a_entry = entry;
 
@@ -220,10 +215,14 @@ unexec (new_name, a_name, bndry, bss_start, entry)
     unsigned long daddr = N_DATADDR (ohdr);
     unsigned long rel, erel;
 #ifdef SUNOS4
+#ifdef SUNOS4_SHARED_LIBRARIES
     extern struct link_dynamic _DYNAMIC;
 
     /*  SunOS4.x's ld_rel is relative to N_TXTADDR. */
-    if (_DYNAMIC.ld_version < 2)
+    if (!ohdr.a_dynamic)
+      /* This was statically linked.  */
+      rel = erel = 0;
+    else if (_DYNAMIC.ld_version < 2)
       {
 	rel = _DYNAMIC.ld_un.ld_1->ld_rel + N_TXTADDR (ohdr);
 	erel = _DYNAMIC.ld_un.ld_1->ld_hash + N_TXTADDR (ohdr);
@@ -233,14 +232,17 @@ unexec (new_name, a_name, bndry, bss_start, entry)
 	rel = _DYNAMIC.ld_un.ld_2->ld_rel + N_TXTADDR (ohdr);
 	erel = _DYNAMIC.ld_un.ld_2->ld_hash + N_TXTADDR (ohdr);
       }
+#else /* not SUNOS4_SHARED_LIBRARIES */
+    rel = erel = 0;
+#endif /* not SUNOS4_SHARED_LIBRARIES */
 #ifdef sparc
 #define REL_INFO_TYPE		struct reloc_info_sparc
 #else
-#define REL_INFO_TYPE		struct reloc_info_m68k
+#define REL_INFO_TYPE		struct relocation_info
 #endif /* sparc */
 #define REL_TARGET_ADDRESS(r)	(((REL_INFO_TYPE *)(r))->r_address)
 #endif /* SUNOS4 */
-#ifdef __FreeBSD__
+#if defined (__FreeBSD__) || defined (__NetBSD__)
     extern struct _dynamic _DYNAMIC;
 
     /*  FreeBSD's LD_REL is a virtual address itself. */
@@ -283,7 +285,7 @@ run_time_remap (progname)
     return;
 
   /* Restore the break */
-  brk (Brk);
+  brk ((char *) brk_value);
 
   /*  If nothing to remap:  we are done! */
   if (rd_only_len == 0)
@@ -362,7 +364,7 @@ is_it (filename)
 	       * should the shared library decide to indirect through
 	       * addresses in the data segment not part of __DYNAMIC
 	       */
-	      mmap (data_start, rd_only_len, PROT_READ | PROT_EXEC,
+	      mmap ((char *) data_start, rd_only_len, PROT_READ | PROT_EXEC,
 		    MAP_FILE | MAP_SHARED | MAP_FIXED, fd,
 		    N_DATOFF (hdr) + data_start - N_DATADDR (hdr));
 	      close (fd);
