@@ -1,5 +1,6 @@
 /* Definitions and headers for communication with X protocol.
-   Copyright (C) 1989, 1993, 1994 Free Software Foundation, Inc.
+   Copyright (C) 1989, 1993, 1994, 1998, 1999, 2000, 2001, 2001
+   Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -77,8 +78,7 @@ Boston, MA 02111-1307, USA.  */
 #define HAVE_X11R4
 #endif
 
-#ifdef HAVE_X11R5
-#define HAVE_X_I18N
+#ifdef HAVE_X_I18N
 #include <X11/Xlocale.h>
 #endif
 
@@ -107,31 +107,12 @@ Boston, MA 02111-1307, USA.  */
    | EnterWindowMask		\
    | VisibilityChangeMask)
 
-/* This checks to make sure we have a display.  */
-extern void check_x ();
-
-extern struct frame *x_window_to_frame ();
-
-#ifdef USE_X_TOOLKIT
-extern struct frame *x_any_window_to_frame ();
-extern struct frame *x_non_menubar_window_to_frame ();
-extern struct frame *x_top_window_to_frame ();
-#endif
-
-extern Visual *select_visual ();
-
-enum text_cursor_kinds {
-  filled_box_cursor, hollow_box_cursor, bar_cursor
-};
-
-/* This data type is used for the font_table field
-   of struct x_display_info.  */
-
-struct font_info
+enum text_cursor_kinds
 {
-  XFontStruct *font;
-  char *name;
-  char *full_name;
+  NO_CURSOR = -1,
+  FILLED_BOX_CURSOR,
+  HOLLOW_BOX_CURSOR,
+  BAR_CURSOR
 };
 
 /* Structure recording X pixmap and reference count.
@@ -153,32 +134,51 @@ struct x_display_info
 {
   /* Chain of all x_display_info structures.  */
   struct x_display_info *next;
+  
   /* Connection number (normally a file descriptor number).  */
   int connection;
+  
   /* This says how to access this display in Xlib.  */
   Display *display;
+  
   /* This is a cons cell of the form (NAME . FONT-LIST-CACHE).
      The same cons cell also appears in x_display_name_list.  */
   Lisp_Object name_list_element;
+  
   /* Number of frames that are on this display.  */
   int reference_count;
+  
   /* The Screen this connection is connected to.  */
   Screen *screen;
+
+  /* Dots per inch of the screen.  */
+  double resx, resy;
+  
   /* The Visual being used for this display.  */
   Visual *visual;
+
+  /* THe colormap being used.  */
+  Colormap cmap;
+  
   /* Number of panes on this screen.  */
   int n_planes;
+  
   /* Dimensions of this screen.  */
   int height, width;
+  
   /* Mask of things that cause the mouse to be grabbed.  */
   int grabbed;
+  
   /* Emacs bitmap-id of the default icon bitmap for this frame.
      Or -1 if none has been allocated yet.  */
   int icon_bitmap_id;
+  
   /* The root window of this screen.  */
   Window root_window;
+  
   /* The cursor to use for vertical scroll bars.  */
   Cursor vertical_scroll_bar_cursor;
+  
   /* X Resource data base */
   XrmDatabase xrdb;
 
@@ -188,18 +188,28 @@ struct x_display_info
   /* The current capacity of x_font_table.  */
   int font_table_size;
 
+  /* Minimum width over all characters in all fonts in font_table.  */
+  int smallest_char_width;
+
+  /* Minimum font height over all fonts in font_table.  */
+  int smallest_font_height;
+
   /* Reusable Graphics Context for drawing a cursor in a non-default face. */
   GC scratch_cursor_gc;
 
-  /* These variables describe the range of text currently shown
-     in its mouse-face, together with the window they apply to.
-     As long as the mouse stays within this range, we need not
-     redraw anything on its account.  */
+  /* These variables describe the range of text currently shown in its
+     mouse-face, together with the window they apply to.  As long as
+     the mouse stays within this range, we need not redraw anything on
+     its account.  Rows and columns are glyph matrix positions in
+     MOUSE_FACE_WINDOW.  */
   int mouse_face_beg_row, mouse_face_beg_col;
+  int mouse_face_beg_x, mouse_face_beg_y;
   int mouse_face_end_row, mouse_face_end_col;
+  int mouse_face_end_x, mouse_face_end_y;
   int mouse_face_past_end;
   Lisp_Object mouse_face_window;
   int mouse_face_face_id;
+  Lisp_Object mouse_face_overlay;
 
   /* 1 if a mouse motion event came and we didn't handle it right away because
      gc was in progress.  */
@@ -213,11 +223,13 @@ struct x_display_info
   /* Nonzero means defer mouse-motion highlighting.  */
   int mouse_face_defer;
 
+  int mouse_face_image_state;
+
   char *x_id_name;
 
   /* The number of fonts actually stored in x_font_table.
-     font_table[n] is used and valid iff 0 <= n < n_fonts.
-     0 <= n_fonts <= font_table_size.  */
+     font_table[n] is used and valid iff 0 <= n < n_fonts.  0 <=
+     n_fonts <= font_table_size and font_table[i].name != 0.  */
   int n_fonts;
 
   /* Pointer to bitmap records.  */
@@ -255,22 +267,40 @@ struct x_display_info
 
   /* Communication with window managers.  */
   Atom Xatom_wm_protocols;
+  
   /* Kinds of protocol things we may receive.  */
   Atom Xatom_wm_take_focus;
   Atom Xatom_wm_save_yourself;
   Atom Xatom_wm_delete_window;
+  
   /* Atom for indicating window state to the window manager.  */
   Atom Xatom_wm_change_state;
+  
   /* Other WM communication */
   Atom Xatom_wm_configure_denied; /* When our config request is denied */
   Atom Xatom_wm_window_moved;     /* When the WM moves us.  */
+  
   /* EditRes protocol */
   Atom Xatom_editres;
 
   /* More atoms, which are selection types.  */
   Atom Xatom_CLIPBOARD, Xatom_TIMESTAMP, Xatom_TEXT, Xatom_DELETE,
+  Xatom_COMPOUND_TEXT,
   Xatom_MULTIPLE, Xatom_INCR, Xatom_EMACS_TMP, Xatom_TARGETS, Xatom_NULL,
   Xatom_ATOM_PAIR;
+
+  /* More atoms for font properties.  The last three are private
+     properties, see the comments in src/fontset.h.  */
+  Atom Xatom_PIXEL_SIZE,
+  Xatom_MULE_BASELINE_OFFSET, Xatom_MULE_RELATIVE_COMPOSE,
+  Xatom_MULE_DEFAULT_ASCENT;
+
+  /* More atoms for Ghostscript support.  */
+  Atom Xatom_DONE, Xatom_PAGE;
+
+  /* Atom used in toolkit scroll bar client messages.  */
+  Atom Xatom_Scrollbar;
+
 #ifdef MULTI_KBOARD
   struct kboard *kboard;
 #endif
@@ -295,8 +325,40 @@ struct x_display_info
      frame.  It differs from x_focus_frame when we're using a global
      minibuffer.  */
   struct frame *x_highlight_frame;
+
+  /* The null pixel used for filling a character background with
+     background color of a gc.  */
+  Pixmap null_pixel;
+
+  /* The gray pixmap.  */
+  Pixmap gray;
+
+  /* Cache of images.  */
+  struct image_cache *image_cache;
+
+#ifdef HAVE_X_I18N
+  /* XIM (X Input method).  */
+  XIM xim;
+  XIMStyles *xim_styles;
+#endif
+
+  /* If non-null, a cache of the colors in the color map.  Don't
+     use this directly, call x_color_cells instead.  */
+  XColor *color_cells;
+  int ncolor_cells;
 };
 
+/* This checks to make sure we have a display.  */
+extern void check_x P_ ((void));
+
+extern struct frame *x_window_to_frame P_ ((struct x_display_info *, int));
+
+#ifdef USE_X_TOOLKIT
+extern struct frame *x_any_window_to_frame P_ ((struct x_display_info *, int));
+extern struct frame *x_non_menubar_window_to_frame P_ ((struct x_display_info *, int));
+extern struct frame *x_top_window_to_frame P_ ((struct x_display_info *, int));
+#endif
+
 /* This is a chain of structures for all the X displays currently in use.  */
 extern struct x_display_info *x_display_list;
 
@@ -306,10 +368,23 @@ extern struct x_display_info *x_display_list;
    FONT-LIST-CACHE records previous values returned by x-list-fonts.  */
 extern Lisp_Object x_display_name_list;
 
-extern struct x_display_info *x_display_info_for_display ();
-extern struct x_display_info *x_display_info_for_name ();
+/* Regexp matching a font name whose width is the same as `PIXEL_SIZE'.  */
+extern Lisp_Object Vx_pixel_size_width_font_regexp;
 
-extern struct x_display_info *x_term_init ();
+/* A flag to control how to display unibyte 8-bit character.  */
+extern int unibyte_display_via_language_environment;
+
+extern struct x_display_info *x_display_info_for_display P_ ((Display *));
+extern struct x_display_info *x_display_info_for_name P_ ((Lisp_Object));
+
+extern struct x_display_info *x_term_init P_ ((Lisp_Object, char *, char *));
+
+extern Lisp_Object x_list_fonts P_ ((struct frame *, Lisp_Object, int, int));
+extern void select_visual P_ ((struct x_display_info *));
+extern struct font_info *x_get_font_info P_ ((struct frame *f, int));
+extern struct font_info *x_load_font P_ ((struct frame *, char *, int));
+extern struct font_info *x_query_font P_ ((struct frame *, char *));
+extern void x_find_ccl_program P_ ((struct font_info *));
 
 /* Each X frame object points to its own struct x_output object
    in the output_data.x field.  The x_output structure contains
@@ -380,8 +455,16 @@ struct x_output
      icon. */
   int icon_bitmap;
 
+  /* Default ASCII font of this frame.  */
   XFontStruct *font;
 
+  /* The baseline offset of the default ASCII font.  */
+  int baseline_offset;
+
+  /* If a fontset is specified for this frame instead of font, this
+     value contains an ID of the fontset, else -1.  */
+  int fontset;
+
   /* Pixel values used for various purposes.
      border_pixel may be -1 meaning use a gray tile.  */
   unsigned long background_pixel;
@@ -391,11 +474,29 @@ struct x_output
   unsigned long mouse_pixel;
   unsigned long cursor_foreground_pixel;
 
+  /* Foreground color for scroll bars.  A value of -1 means use the
+     default (black for non-toolkit scroll bars).  */
+  unsigned long scroll_bar_foreground_pixel;
+  
+  /* Background color for scroll bars.  A value of -1 means use the
+     default (background color of the frame for non-toolkit scroll
+     bars).  */
+  unsigned long scroll_bar_background_pixel;
+
   /* Descriptor for the cursor in use for this window.  */
   Cursor text_cursor;
   Cursor nontext_cursor;
   Cursor modeline_cursor;
   Cursor cross_cursor;
+  Cursor hourglass_cursor;
+  Cursor horizontal_drag_cursor;
+
+  /* Window whose cursor is hourglass_cursor.  This window is temporarily
+     mapped to display an hourglass cursor.  */
+  Window hourglass_window;
+  
+  /* Non-zero means hourglass cursor is currently displayed.  */
+  unsigned hourglass_p : 1;
 
   /* Flag to set when the X window needs to be completely repainted.  */
   int needs_exposure;
@@ -428,20 +529,10 @@ struct x_output
      scroll bars, in pixels.  */
   int vertical_scroll_bar_extra;
 
-  /* Table of parameter faces for this frame.  Any X resources (pixel
-     values, fonts) referred to here have been allocated explicitly
-     for this face, and should be freed if we change the face.  */
-  struct face **param_faces;
-  int n_param_faces;
-
-  /* Table of computed faces for this frame.  These are the faces
-     whose indexes go into the upper bits of a glyph, computed by
-     combining the parameter faces specified by overlays, text
-     properties, and what have you.  The X resources mentioned here
-     are all shared with parameter faces.  */
-  struct face **computed_faces;
-  int n_computed_faces;		/* How many are valid */
-  int size_computed_faces;	/* How many are allocated */
+  /* The extra width currently allotted for the areas in which
+     truncation marks, continuation marks, and overlay arrows are
+     displayed.  */
+  int flags_areas_extra;
 
   /* This is the gravity value for the specified window position.  */
   int win_gravity;
@@ -468,35 +559,57 @@ struct x_output
   /* Nonzero means tried already to make this frame visible.  */
   char asked_for_visible;
 
+  /* Nonzero if this frame was ever previously visible.  */
+  char has_been_visible;
+
 #ifdef HAVE_X_I18N
-  /* Input method. */
-  XIM xim;
   /* Input context (currently, this means Compose key handler setup).  */
   XIC xic;
+  XIMStyle xic_style;
+  XFontSet xic_xfs;
 #endif
+
+  /* Relief GCs, colors etc.  */
+  struct relief
+  {
+    GC gc;
+    unsigned long pixel;
+    int allocated_p;
+  }
+  black_relief, white_relief;
+
+  /* The background for which the above relief GCs were set up.
+     They are changed only when a different background is involved.  */
+  unsigned long relief_background;
+
+  /* Xt waits for a ConfigureNotify event from the window manager in
+     EmacsFrameSetCharSize when the shell widget is resized.  For some
+     window managers like fvwm2 2.2.5 and KDE 2.1 this event doesn't
+     arrive for an unknown reason and Emacs hangs in Xt.  If this is
+     zero, tell Xt not to wait.  */
+  int wait_for_wm;
 };
 
-/* Get at the computed faces of an X window frame.  */
-#define FRAME_PARAM_FACES(f) ((f)->output_data.x->param_faces)
-#define FRAME_N_PARAM_FACES(f) ((f)->output_data.x->n_param_faces)
-#define FRAME_DEFAULT_PARAM_FACE(f) (FRAME_PARAM_FACES (f)[0])
-#define FRAME_MODE_LINE_PARAM_FACE(f) (FRAME_PARAM_FACES (f)[1])
-
-#define FRAME_COMPUTED_FACES(f) ((f)->output_data.x->computed_faces)
-#define FRAME_N_COMPUTED_FACES(f) ((f)->output_data.x->n_computed_faces)
-#define FRAME_SIZE_COMPUTED_FACES(f) ((f)->output_data.x->size_computed_faces)
-#define FRAME_DEFAULT_FACE(f) ((f)->output_data.x->computed_faces[0])
-#define FRAME_MODE_LINE_FACE(f) ((f)->output_data.x->computed_faces[1])
-
-/* Return the window associated with the frame F.  */
+/* Return the X window used for displaying data in frame F.  */
 #define FRAME_X_WINDOW(f) ((f)->output_data.x->window_desc)
 
-#define FRAME_FOREGROUND_PIXEL(f) ((f)->output_data.x->foreground_pixel)
-#define FRAME_BACKGROUND_PIXEL(f) ((f)->output_data.x->background_pixel)
+/* Return the outermost X window associated with the frame F.  */
+#ifdef USE_X_TOOLKIT
+#define FRAME_OUTER_WINDOW(f) (XtWindow ((f)->output_data.x->widget))
+#else
+#define FRAME_OUTER_WINDOW(f) (FRAME_X_WINDOW (f))
+#endif
+
 #define FRAME_FONT(f) ((f)->output_data.x->font)
+#define FRAME_FONTSET(f) ((f)->output_data.x->fontset)
 #define FRAME_INTERNAL_BORDER_WIDTH(f) ((f)->output_data.x->internal_border_width)
+#define FRAME_MENUBAR_HEIGHT(f) ((f)->output_data.x->menubar_height)
 #define FRAME_LINE_HEIGHT(f) ((f)->output_data.x->line_height)
 
+/* Width of the default font of frame F.  Must be defined by each
+   terminal specific header.  */
+#define FRAME_DEFAULT_FONT_WIDTH(F) 	FONT_WIDTH (FRAME_FONT (F))
+
 /* This gives the x_display_info structure for the display F is on.  */
 #define FRAME_X_DISPLAY_INFO(f) ((f)->output_data.x->display_info)
 
@@ -505,6 +618,16 @@ struct x_output
 
 /* This is the `Screen *' which frame F is on.  */
 #define FRAME_X_SCREEN(f) (FRAME_X_DISPLAY_INFO (f)->screen)
+#define FRAME_X_SCREEN_NUMBER(f) XScreenNumberOfScreen (FRAME_X_SCREEN (f))
+
+/* This is the Visual which frame F is on.  */
+#define FRAME_X_VISUAL(f) FRAME_X_DISPLAY_INFO (f)->visual
+
+/* This is the Colormap which frame F uses.  */
+#define FRAME_X_COLORMAP(f) FRAME_X_DISPLAY_INFO (f)->cmap
+
+/* This is the 'font_info *' which frame F has.  */
+#define FRAME_X_FONT_TABLE(f) (FRAME_X_DISPLAY_INFO (f)->font_table)
 
 /* These two really ought to be called FRAME_PIXEL_{WIDTH,HEIGHT}.  */
 #define PIXEL_WIDTH(f) ((f)->output_data.x->pixel_width)
@@ -512,8 +635,61 @@ struct x_output
 
 #define FRAME_DESIRED_CURSOR(f) ((f)->output_data.x->desired_cursor)
 
-#define FRAME_XIM(f) ((f)->output_data.x->xim)
 #define FRAME_XIC(f) ((f)->output_data.x->xic)
+#define FRAME_X_XIM(f) (FRAME_X_DISPLAY_INFO (f)->xim)
+#define FRAME_X_XIM_STYLES(f) (FRAME_X_DISPLAY_INFO (f)->xim_styles)
+#define FRAME_XIC_STYLE(f) ((f)->output_data.x->xic_style)
+#define FRAME_XIC_FONTSET(f) ((f)->output_data.x->xic_xfs)
+
+/* Value is the smallest width of any character in any font on frame F.  */
+
+#define FRAME_SMALLEST_CHAR_WIDTH(F) \
+     FRAME_X_DISPLAY_INFO(F)->smallest_char_width
+
+/* Value is the smallest height of any font on frame F.  */
+
+#define FRAME_SMALLEST_FONT_HEIGHT(F) \
+     FRAME_X_DISPLAY_INFO(F)->smallest_font_height
+
+/* Return a pointer to the image cache of frame F.  */
+
+#define FRAME_X_IMAGE_CACHE(F) FRAME_X_DISPLAY_INFO ((F))->image_cache
+
+
+/* Pixel width of the bitmaps drawn to indicate truncation,
+   continuation etc.  */
+
+#define FRAME_FLAGS_BITMAP_WIDTH(f)	8
+#define FRAME_FLAGS_BITMAP_HEIGHT(f)	8
+
+/* Total width of a areas reserved for drawing truncation bitmaps,
+   continuation bitmaps and alike.  The width is in canonical char
+   units of the frame.  This must currently be the case because window
+   sizes aren't pixel values.  If it weren't the case, we wouldn't be
+   able to split windows horizontally nicely.  */
+
+#define FRAME_X_FLAGS_AREA_COLS(F)					\
+     ((2 * FRAME_FLAGS_BITMAP_WIDTH ((F)) + CANON_X_UNIT ((F)) - 1)	\
+      / CANON_X_UNIT ((F)))
+
+/* Total width of flags areas in pixels.  */
+
+#define FRAME_X_FLAGS_AREA_WIDTH(F) \
+     (FRAME_X_FLAGS_AREA_COLS ((F)) * CANON_X_UNIT ((F)))
+
+/* Pixel-width of the left flags area.  */
+
+#define FRAME_X_LEFT_FLAGS_AREA_WIDTH(F) \
+     (FRAME_X_FLAGS_AREA_WIDTH (F) / 2)
+
+/* Pixel-width of the right flags area.  Note that we are doing
+   integer arithmetic here, so don't loose a pixel if the total
+   width is an odd number.  */
+
+#define FRAME_X_RIGHT_FLAGS_AREA_WIDTH(F) 	\
+     (FRAME_X_FLAGS_AREA_WIDTH (F) - FRAME_X_FLAGS_AREA_WIDTH (F) / 2)
+
+
 
 /* X-specific scroll bar stuff.  */
 
@@ -525,8 +701,8 @@ struct x_output
    We use struct scroll_bar as a template for accessing fields of the
    vector.  */
 
-struct scroll_bar {
-
+struct scroll_bar
+{
   /* These fields are shared by all vectors.  */
   EMACS_INT size_from_Lisp_Vector_struct;
   struct Lisp_Vector *next_from_Lisp_Vector_struct;
@@ -592,16 +768,29 @@ struct scroll_bar {
 #define SET_SCROLL_BAR_X_WINDOW(ptr, id) \
   (SCROLL_BAR_UNPACK ((ptr)->x_window_low, (ptr)->x_window_high, (int) id))
 
+/* Extract the X widget of the scroll bar from a struct scroll_bar.
+   XtWindowToWidget should be fast enough since Xt uses a hash table
+   to map windows to widgets.  */
+
+#define SCROLL_BAR_X_WIDGET(dpy, ptr) \
+  XtWindowToWidget (dpy, SCROLL_BAR_X_WINDOW (ptr))
+
+/* Store a widget id in a struct scroll_bar.  */
+
+#define SET_SCROLL_BAR_X_WIDGET(ptr, w)		\
+  do {						\
+    Window window = XtWindow (w);		\
+    SET_SCROLL_BAR_X_WINDOW (ptr, window);	\
+} while (0)
 
-/* Return the outside pixel height for a vertical scroll bar HEIGHT
-   rows high on frame F.  */
-#define VERTICAL_SCROLL_BAR_PIXEL_HEIGHT(f, height) \
-  ((height) * (f)->output_data.x->line_height)
 
 /* Return the inside width of a vertical scroll bar, given the outside
    width.  */
-#define VERTICAL_SCROLL_BAR_INSIDE_WIDTH(width) \
-  ((width) - VERTICAL_SCROLL_BAR_LEFT_BORDER - VERTICAL_SCROLL_BAR_RIGHT_BORDER)
+#define VERTICAL_SCROLL_BAR_INSIDE_WIDTH(f, width) \
+  ((width) \
+   - VERTICAL_SCROLL_BAR_LEFT_BORDER \
+   - VERTICAL_SCROLL_BAR_RIGHT_BORDER \
+   - VERTICAL_SCROLL_BAR_WIDTH_TRIM * 2)
 
 /* Return the length of the rectangle within which the top of the
    handle must stay.  This isn't equivalent to the inside height,
@@ -610,12 +799,12 @@ struct scroll_bar {
    This is the real range of motion for the scroll bar, so when we're
    scaling buffer positions to scroll bar positions, we use this, not
    VERTICAL_SCROLL_BAR_INSIDE_HEIGHT.  */
-#define VERTICAL_SCROLL_BAR_TOP_RANGE(height) \
-  (VERTICAL_SCROLL_BAR_INSIDE_HEIGHT (height) - VERTICAL_SCROLL_BAR_MIN_HANDLE)
+#define VERTICAL_SCROLL_BAR_TOP_RANGE(f, height) \
+  (VERTICAL_SCROLL_BAR_INSIDE_HEIGHT (f, height) - VERTICAL_SCROLL_BAR_MIN_HANDLE)
 
 /* Return the inside height of vertical scroll bar, given the outside
    height.  See VERTICAL_SCROLL_BAR_TOP_RANGE too.  */
-#define VERTICAL_SCROLL_BAR_INSIDE_HEIGHT(height) \
+#define VERTICAL_SCROLL_BAR_INSIDE_HEIGHT(f, height) \
   ((height) - VERTICAL_SCROLL_BAR_TOP_BORDER - VERTICAL_SCROLL_BAR_BOTTOM_BORDER)
 
 
@@ -638,6 +827,10 @@ struct scroll_bar {
 /* Minimum lengths for scroll bar handles, in pixels.  */
 #define VERTICAL_SCROLL_BAR_MIN_HANDLE (5)
 
+/* Trimming off a few pixels from each side prevents
+   text from glomming up against the scroll bar */
+#define VERTICAL_SCROLL_BAR_WIDTH_TRIM (0)
+
 
 /* Manipulating pixel sizes and character sizes.
    Knowledge of which factors affect the overall size of the window should
@@ -657,6 +850,7 @@ struct scroll_bar {
 #define CHAR_TO_PIXEL_WIDTH(f, width) \
   (CHAR_TO_PIXEL_COL (f, width) \
    + (f)->output_data.x->vertical_scroll_bar_extra \
+   + (f)->output_data.x->flags_areas_extra \
    + (f)->output_data.x->internal_border_width)
 #define CHAR_TO_PIXEL_HEIGHT(f, height) \
   (CHAR_TO_PIXEL_ROW (f, height) \
@@ -677,10 +871,12 @@ struct scroll_bar {
 #define PIXEL_TO_CHAR_WIDTH(f, width) \
   (PIXEL_TO_CHAR_COL (f, ((width) \
 			  - (f)->output_data.x->internal_border_width \
+			  - (f)->output_data.x->flags_areas_extra \
 			  - (f)->output_data.x->vertical_scroll_bar_extra)))
 #define PIXEL_TO_CHAR_HEIGHT(f, height) \
   (PIXEL_TO_CHAR_ROW (f, ((height) \
 			  - (f)->output_data.x->internal_border_width)))
+
 
 /* If a struct input_event has a kind which is selection_request_event
    or selection_clear_event, then its contents are really described
@@ -716,45 +912,159 @@ struct selection_input_event
   (((struct selection_input_event *) (eventp))->time)
 
 
-/* Interface to the face code functions.  */
+struct window;
+struct glyph_matrix;
+struct frame;
+struct input_event;
+struct face;
+struct image;
 
-/* Create the first two computed faces for a frame -- the ones that
-   have GC's.  */
-extern void init_frame_faces (/* FRAME_PTR */);
+/* From xselect.c.  */
 
-/* Free the resources for the faces associated with a frame.  */
-extern void free_frame_faces (/* FRAME_PTR */);
+void x_handle_selection_notify P_ ((XSelectionEvent *));
+void x_handle_property_notify P_ ((XPropertyEvent *));
 
-/* Given a computed face, find or make an equivalent display face
-   in face_vector, and return a pointer to it.  */
-extern struct face *intern_face (/* FRAME_PTR, struct face * */);
+/* From xfns.c.  */
 
-/* Given a frame and a face name, return the face's ID number, or
-   zero if it isn't a recognized face name.  */
-extern int face_name_id_number (/* FRAME_PTR, Lisp_Object */);
+Lisp_Object display_x_get_resource P_ ((struct x_display_info *,
+					Lisp_Object, Lisp_Object,
+					Lisp_Object, Lisp_Object));
+struct frame *check_x_frame P_ ((Lisp_Object));
+EXFUN (Fx_display_color_p, 1);
+EXFUN (Fx_display_grayscale_p, 1);
+int image_ascent P_ ((struct image *, struct face *));
+extern void x_free_gcs P_ ((struct frame *));
 
-/* Return non-zero if FONT1 and FONT2 have the same size bounding box.
-   We assume that they're both character-cell fonts.  */
-extern int same_size_fonts (/* XFontStruct *, XFontStruct * */);
+/* From xrdb.c.  */
 
-/* Recompute the GC's for the default and modeline faces.
-   We call this after changing frame parameters on which those GC's
-   depend.  */
-extern void recompute_basic_faces (/* FRAME_PTR */);
+char *x_get_string_resource P_ ((XrmDatabase, char *, char *));
+char *x_get_customization_string P_ ((XrmDatabase, char *, char *));
+XrmDatabase x_load_resources P_ ((Display *, char *, char *, char *));
+int x_get_resource P_ ((XrmDatabase, char *, char *,
+			XrmRepresentation, XrmValue *));
+void x_delete_display P_ ((struct x_display_info *));
+void x_make_frame_visible P_ ((struct frame *));
+void x_iconify_frame P_ ((struct frame *));
+void x_wm_set_size_hint P_ ((struct frame *, long, int));
+void x_set_offset P_ ((struct frame *, int, int, int));
+void x_wm_set_icon_position P_ ((struct frame *, int, int));
+int x_catch_errors P_ ((Display *));
+int x_had_errors_p P_ ((Display *));
+void x_uncatch_errors P_ ((Display *, int));
+void x_check_errors P_ ((Display *, char *));
+int x_text_icon P_ ((struct frame *, char *));
+int x_bitmap_icon P_ ((struct frame *, Lisp_Object));
+void x_set_window_size P_ ((struct frame *, int, int, int));
+void x_wm_set_window_state P_ ((struct frame *, int));
+int x_alloc_nearest_color P_ ((struct frame *, Colormap, XColor *));
 
-/* Return the face ID associated with a buffer position POS.  Store
-   into *ENDPTR the next position at which a different face is
-   needed.  This does not take account of glyphs that specify their
-   own face codes.  F is the frame in use for display, and W is a
-   window displaying the current buffer.
+extern void pixel_to_glyph_coords P_ ((struct frame *, int, int,
+				       int *, int *, XRectangle *, int));
 
-   REGION_BEG, REGION_END delimit the region, so it can be highlighted.  */
-extern int compute_char_face (/* FRAME_PTR frame,
-				 struct window *w,
-				 int pos,
-				 int region_beg, int region_end,
-				 int *endptr */);
-/* Return the face ID to use to display a special glyph which selects
-   FACE_CODE as the face ID, assuming that ordinarily the face would
-   be BASIC_FACE.  F is the frame.  */
-extern int compute_glyph_face (/* FRAME_PTR, int */);
+/* Defined in xterm.c */
+
+extern void cancel_mouse_face P_ ((struct frame *));
+extern void x_scroll_bar_clear P_ ((struct frame *));
+extern void x_start_queuing_selection_requests P_ ((Display *));
+extern void x_stop_queuing_selection_requests P_ ((Display *));
+extern void x_update_cursor P_ ((struct frame *, int));
+extern int x_text_icon P_ ((struct frame *, char *));
+extern int x_bitmap_icon P_ ((struct frame *, Lisp_Object));
+extern int x_catch_errors P_ ((Display *));
+extern void x_check_errors P_ ((Display *, char *));
+extern int x_had_errors_p P_ ((Display *));
+extern void x_uncatch_errors P_ ((Display *, int));
+extern Lisp_Object x_new_font P_ ((struct frame *, char *));
+extern Lisp_Object x_new_fontset P_ ((struct frame *, char *));
+extern void x_set_offset P_ ((struct frame *, int, int, int));
+extern void x_set_window_size P_ ((struct frame *, int, int, int));
+extern void x_set_mouse_position P_ ((struct frame *, int, int));
+extern void x_set_mouse_pixel_position P_ ((struct frame *, int, int));
+extern void x_raise_frame P_ ((struct frame *));
+extern void x_lower_frame P_ ((struct frame *));
+extern void x_make_frame_visible P_ ((struct frame *));
+extern void x_make_frame_invisible P_ ((struct frame *));
+extern void x_iconify_frame P_ ((struct frame *));
+extern void x_free_frame_resources P_ ((struct frame *));
+extern void x_destroy_window P_ ((struct frame *));
+extern void x_wm_set_size_hint P_ ((struct frame *, long, int));
+extern void x_wm_set_window_state P_ ((struct frame *, int));
+extern void x_wm_set_icon_pixmap P_ ((struct frame *, int));
+extern void x_wm_set_icon_position P_ ((struct frame *, int, int));
+extern void x_delete_display P_ ((struct x_display_info *));
+extern void x_initialize P_ ((void));
+extern void x_display_cursor P_ ((struct window *, int, int, int, int, int));
+extern void x_update_cursor P_ ((struct frame *, int));
+extern unsigned long x_copy_color P_ ((struct frame *, unsigned long));
+#ifdef USE_X_TOOLKIT
+extern XtAppContext Xt_app_con;
+#endif
+extern void x_query_colors P_ ((struct frame *f, XColor *, int));
+extern void x_query_color P_ ((struct frame *f, XColor *));
+extern void x_clear_area P_ ((Display *, Window, int, int, int, int, int));
+
+/* Defined in xselect.c */
+
+extern void x_handle_property_notify P_ ((XPropertyEvent *));
+extern void x_handle_selection_notify P_ ((XSelectionEvent *));
+extern void x_handle_selection_request P_ ((struct input_event *));
+extern void x_handle_selection_clear P_ ((struct input_event *));
+extern void x_clear_frame_selections P_ ((struct frame *));
+
+/* Defined in xfns.c */
+
+extern int have_menus_p P_ ((void));
+extern int x_bitmap_height P_ ((struct frame *, int));
+extern int x_bitmap_width P_ ((struct frame *, int));
+extern int x_bitmap_pixmap P_ ((struct frame *, int));
+extern void x_reference_bitmap P_ ((struct frame *, int));
+extern int x_create_bitmap_from_data P_ ((struct frame *, char *,
+					  unsigned int, unsigned int));
+extern int x_create_bitmap_from_file P_ ((struct frame *, Lisp_Object));
+extern void x_destroy_bitmap P_ ((struct frame *, int));
+extern void x_set_frame_parameters P_ ((struct frame *, Lisp_Object));
+extern void x_real_positions P_ ((struct frame *, int *, int *));
+extern void x_report_frame_params P_ ((struct frame *, Lisp_Object *));
+extern int defined_color P_ ((struct frame *, char *, XColor *, int));
+extern void x_set_border_pixel P_ ((struct frame *, int));
+extern void x_set_menu_bar_lines P_ ((struct frame *, Lisp_Object, Lisp_Object));
+extern unsigned char * x_encode_text P_ ((Lisp_Object, Lisp_Object, int,
+					  int *, int *));
+extern void x_implicitly_set_name P_ ((struct frame *, Lisp_Object, Lisp_Object));
+extern void create_frame_xic P_ ((struct frame *));
+extern void destroy_frame_xic P_ ((struct frame *));
+extern void xic_set_preeditarea P_ ((struct window *, int, int));
+extern void xic_set_statusarea P_ ((struct frame *));
+extern void xic_set_xfontset P_ ((struct frame *, char *));
+extern int x_pixel_width P_ ((struct frame *));
+extern int x_pixel_height P_ ((struct frame *));
+extern int x_char_width P_ ((struct frame *));
+extern int x_char_height P_ ((struct frame *));
+extern int x_screen_planes P_ ((struct frame *));
+extern void x_sync P_ ((struct frame *));
+extern enum text_cursor_kinds x_specified_cursor_type P_ ((Lisp_Object, int *));
+extern int x_defined_color P_ ((struct frame *, char *, XColor *, int));
+#ifdef HAVE_X_I18N
+extern void free_frame_xic P_ ((struct frame *));
+#endif
+extern void x_set_tool_bar_lines P_ ((struct frame *, Lisp_Object, Lisp_Object));
+
+/* Defined in xfaces.c */
+
+extern int frame_update_line_height P_ ((struct frame *));
+extern int compute_glyph_face P_ ((struct frame *, int, int));
+extern int compute_glyph_face_1 P_ ((struct frame *, Lisp_Object, int));
+extern void x_free_dpy_colors P_ ((Display *, Screen *, Colormap,
+				   unsigned long *, int));
+
+/* Defined in xmenu.c */
+
+extern void x_activate_menubar P_ ((struct frame *));
+extern int popup_activated P_ ((void));
+extern void initialize_frame_menubar P_ ((struct frame *));
+
+/* Defined in widget.c */
+
+#ifdef USE_X_TOOLKIT
+extern void widget_store_internal_border P_ ((Widget));
+#endif
